<p><a href="{{ pathto(master_doc) }}">
  <img src="{{ pathto('_images/logo-full.jpg', 1) }}" alt="Logo" style="width:80%;height:80%"/>
</a></p>

<h3>About PyVISA</h3>
Python wrapper for the Virtual Instrument Software Architechture library.
You are currently looking at the documentation of version {{ version }}.
<h3>Other Formats</h3>
<p>
  You can download the documentation in other formats as well:
</p>
<ul>
  <li><a href="https://media.readthedocs.org/pdf/pyvisa/latest/pyvisa.pdf">as PDF</a>
  <li><a href="https://media.readthedocs.org/htmlzip/pyvisa/latest/pyvisa.zip">as ePub</a>
  <li><a href="https://media.readthedocs.org/epub/pyvisa/latest/pyvisa.epub">as zipped HTML</a>
</ul>
<h3>Useful Links</h3>
<ul>
  <li><a href="https://pypi.python.org/pypi/pyvisa/">PyVISA @ PyPI</a></li>
  <li><a href="https://github.com/hgrecco/pyvisa">Code in GitHub</a></li>
  <li><a href="https://github.com/hgrecco/pyvisa/issues">Issue Tracker</a></li>
</ul>

