* {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

article, aside, details, figcaption, figure, footer, header, hgroup, nav, section {
  display: block;
}

audio, canvas, video {
  display: inline-block;
  *display: inline;
  *zoom: 1;
}

audio:not([controls]) {
  display: none;
}

[hidden] {
  display: none;
}

* {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

html {
  font-size: 100%;
  -webkit-text-size-adjust: 100%;
  -ms-text-size-adjust: 100%;
}

body {
  margin: 0;
}

a:hover, a:active {
  outline: 0;
}

abbr[title] {
  border-bottom: 1px dotted;
}

b, strong {
  font-weight: bold;
}

blockquote {
  margin: 0;
}

dfn {
  font-style: italic;
}

hr {
  display: block;
  height: 1px;
  border: 0;
  border-top: 1px solid #cccccc;
  margin: 20px 0;
  padding: 0;
}

ins {
  background: #ffff99;
  color: black;
  text-decoration: none;
}

mark {
  background: yellow;
  color: black;
  font-style: italic;
  font-weight: bold;
}

pre, code, .rst-content tt, kbd, samp {
  font-family: monospace, serif;
  _font-family: "courier new", monospace;
  font-size: 1em;
}

pre {
  white-space: pre;
}

q {
  quotes: none;
}

q:before, q:after {
  content: "";
  content: none;
}

small {
  font-size: 85%;
}

sub, sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline;
}

sup {
  top: -0.5em;
}

sub {
  bottom: -0.25em;
}

ul, ol, dl {
  margin: 0;
  padding: 0;
  list-style: none;
  list-style-image: none;
}

li {
  list-style: none;
}

dd {
  margin: 0;
}

img {
  border: 0;
  -ms-interpolation-mode: bicubic;
  vertical-align: middle;
  max-width: 100%;
}

svg:not(:root) {
  overflow: hidden;
}

figure {
  margin: 0;
}

form {
  margin: 0;
}

fieldset {
  border: 0;
  margin: 0;
  padding: 0;
}

label {
  cursor: pointer;
}

legend {
  border: 0;
  *margin-left: -7px;
  padding: 0;
  white-space: normal;
}

button, input, select, textarea {
  font-size: 100%;
  margin: 0;
  vertical-align: baseline;
  *vertical-align: middle;
}

button, input {
  line-height: normal;
}

button, input[type="button"], input[type="reset"], input[type="submit"] {
  cursor: pointer;
  -webkit-appearance: button;
  *overflow: visible;
}

button[disabled], input[disabled] {
  cursor: default;
}

input[type="checkbox"], input[type="radio"] {
  box-sizing: border-box;
  padding: 0;
  *width: 13px;
  *height: 13px;
}

input[type="search"] {
  -webkit-appearance: textfield;
  -moz-box-sizing: content-box;
  -webkit-box-sizing: content-box;
  box-sizing: content-box;
}

input[type="search"]::-webkit-search-decoration, input[type="search"]::-webkit-search-cancel-button {
  -webkit-appearance: none;
}

button::-moz-focus-inner, input::-moz-focus-inner {
  border: 0;
  padding: 0;
}

textarea {
  overflow: auto;
  vertical-align: top;
  resize: vertical;
}

table {
  border-collapse: collapse;
  border-spacing: 0;
}

td {
  vertical-align: top;
}

.chromeframe {
  margin: 0.2em 0;
  background: #cccccc;
  color: black;
  padding: 0.2em 0;
}

.ir {
  display: block;
  border: 0;
  text-indent: -999em;
  overflow: hidden;
  background-color: transparent;
  background-repeat: no-repeat;
  text-align: left;
  direction: ltr;
  *line-height: 0;
}

.ir br {
  display: none;
}

.hidden {
  display: none !important;
  visibility: hidden;
}

.visuallyhidden {
  border: 0;
  clip: rect(0 0 0 0);
  height: 1px;
  margin: -1px;
  overflow: hidden;
  padding: 0;
  position: absolute;
  width: 1px;
}

.visuallyhidden.focusable:active, .visuallyhidden.focusable:focus {
  clip: auto;
  height: auto;
  margin: 0;
  overflow: visible;
  position: static;
  width: auto;
}

.invisible {
  visibility: hidden;
}

.relative {
  position: relative;
}

big, small {
  font-size: 100%;
}

@media print {
  html, body, section {
    background: none !important;
  }

  * {
    box-shadow: none !important;
    text-shadow: none !important;
    filter: none !important;
    -ms-filter: none !important;
  }

  a, a:visited {
    text-decoration: underline;
  }

  .ir a:after, a[href^="javascript:"]:after, a[href^="#"]:after {
    content: "";
  }

  pre, blockquote {
    page-break-inside: avoid;
  }

  thead {
    display: table-header-group;
  }

  tr, img {
    page-break-inside: avoid;
  }

  img {
    max-width: 100% !important;
  }

  @page {
    margin: 0.5cm;
}

  p, h2, h3 {
    orphans: 3;
    widows: 3;
  }

  h2, h3 {
    page-break-after: avoid;
  }
}
.font-smooth, .wy-alert, .rst-content .note, .rst-content .attention, .rst-content .caution, .rst-content .danger, .rst-content .error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .warning, .rst-content .seealso, .btn, input[type="text"], input[type="password"], input[type="email"], input[type="url"], input[type="date"], input[type="month"], input[type="time"], input[type="datetime"], input[type="datetime-local"], input[type="week"], input[type="number"], input[type="search"], input[type="tel"], input[type="color"], select, textarea, .wy-tag-input-group, .wy-menu-vertical li.on a, .wy-menu-vertical li.current > a, .wy-side-nav-search > a, .wy-side-nav-search .wy-dropdown > a, .wy-nav-top a {
  -webkit-font-smoothing: antialiased;
}

.clearfix {
  *zoom: 1;
}
.clearfix:before, .clearfix:after {
  display: table;
  content: "";
}
.clearfix:after {
  clear: both;
}

/*!
 *  Font Awesome 4.0.3 by @davegandy - http://fontawesome.io - @fontawesome
 *  License - http://fontawesome.io/license (Font: SIL OFL 1.1, CSS: MIT License)
 */
/* FONT PATH
 * -------------------------- */
@font-face {
  font-family: 'FontAwesome';
  src: url("../fonts/fontawesome-webfont.eot?v=4.0.3");
  src: url("../fonts/fontawesome-webfont.eot?#iefix&v=4.0.3") format("embedded-opentype"), url("../fonts/fontawesome-webfont.woff?v=4.0.3") format("woff"), url("../fonts/fontawesome-webfont.ttf?v=4.0.3") format("truetype"), url("../fonts/fontawesome-webfont.svg?v=4.0.3#fontawesomeregular") format("svg");
  font-weight: normal;
  font-style: normal;
}

.fa, .wy-dropdown .caret, .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-tag-input-group .wy-tag .wy-tag-remove, .rst-content .admonition-title, .rst-content h1 .headerlink, .rst-content h2 .headerlink, .rst-content h3 .headerlink, .rst-content h4 .headerlink, .rst-content h5 .headerlink, .rst-content h6 .headerlink, .rst-content dl dt .headerlink, .icon {
  display: inline-block;
  font-family: FontAwesome;
  font-style: normal;
  font-weight: normal;
  line-height: 1;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

/* makes the font 33% larger relative to the icon container */
.fa-lg {
  font-size: 1.33333em;
  line-height: 0.75em;
  vertical-align: -15%;
}

.fa-2x {
  font-size: 2em;
}

.fa-3x {
  font-size: 3em;
}

.fa-4x {
  font-size: 4em;
}

.fa-5x {
  font-size: 5em;
}

.fa-fw {
  width: 1.28571em;
  text-align: center;
}

.fa-ul {
  padding-left: 0;
  margin-left: 2.14286em;
  list-style-type: none;
}
.fa-ul > li {
  position: relative;
}

.fa-li {
  position: absolute;
  left: -2.14286em;
  width: 2.14286em;
  top: 0.14286em;
  text-align: center;
}
.fa-li.fa-lg {
  left: -1.85714em;
}

.fa-border {
  padding: .2em .25em .15em;
  border: solid 0.08em #eeeeee;
  border-radius: .1em;
}

.pull-right {
  float: right;
}

.pull-left {
  float: left;
}

.fa.pull-left, .wy-dropdown .pull-left.caret, .wy-inline-validate.wy-inline-validate-success .pull-left.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .pull-left.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .pull-left.wy-input-context, .wy-inline-validate.wy-inline-validate-info .pull-left.wy-input-context, .wy-tag-input-group .wy-tag .pull-left.wy-tag-remove, .rst-content .pull-left.admonition-title, .rst-content h1 .pull-left.headerlink, .rst-content h2 .pull-left.headerlink, .rst-content h3 .pull-left.headerlink, .rst-content h4 .pull-left.headerlink, .rst-content h5 .pull-left.headerlink, .rst-content h6 .pull-left.headerlink, .rst-content dl dt .pull-left.headerlink, .pull-left.icon {
  margin-right: .3em;
}
.fa.pull-right, .wy-dropdown .pull-right.caret, .wy-inline-validate.wy-inline-validate-success .pull-right.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .pull-right.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .pull-right.wy-input-context, .wy-inline-validate.wy-inline-validate-info .pull-right.wy-input-context, .wy-tag-input-group .wy-tag .pull-right.wy-tag-remove, .rst-content .pull-right.admonition-title, .rst-content h1 .pull-right.headerlink, .rst-content h2 .pull-right.headerlink, .rst-content h3 .pull-right.headerlink, .rst-content h4 .pull-right.headerlink, .rst-content h5 .pull-right.headerlink, .rst-content h6 .pull-right.headerlink, .rst-content dl dt .pull-right.headerlink, .pull-right.icon {
  margin-left: .3em;
}

.fa-spin {
  -webkit-animation: spin 2s infinite linear;
  -moz-animation: spin 2s infinite linear;
  -o-animation: spin 2s infinite linear;
  animation: spin 2s infinite linear;
}

@-moz-keyframes spin {
  0% {
    -moz-transform: rotate(0deg);
  }

  100% {
    -moz-transform: rotate(359deg);
  }
}

@-webkit-keyframes spin {
  0% {
    -webkit-transform: rotate(0deg);
  }

  100% {
    -webkit-transform: rotate(359deg);
  }
}

@-o-keyframes spin {
  0% {
    -o-transform: rotate(0deg);
  }

  100% {
    -o-transform: rotate(359deg);
  }
}

@-ms-keyframes spin {
  0% {
    -ms-transform: rotate(0deg);
  }

  100% {
    -ms-transform: rotate(359deg);
  }
}

@keyframes spin {
  0% {
    transform: rotate(0deg);
  }

  100% {
    transform: rotate(359deg);
  }
}

.fa-rotate-90 {
  filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=$rotation);
  -webkit-transform: rotate(90deg);
  -moz-transform: rotate(90deg);
  -ms-transform: rotate(90deg);
  -o-transform: rotate(90deg);
  transform: rotate(90deg);
}

.fa-rotate-180 {
  filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=$rotation);
  -webkit-transform: rotate(180deg);
  -moz-transform: rotate(180deg);
  -ms-transform: rotate(180deg);
  -o-transform: rotate(180deg);
  transform: rotate(180deg);
}

.fa-rotate-270 {
  filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=$rotation);
  -webkit-transform: rotate(270deg);
  -moz-transform: rotate(270deg);
  -ms-transform: rotate(270deg);
  -o-transform: rotate(270deg);
  transform: rotate(270deg);
}

.fa-flip-horizontal {
  filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=$rotation);
  -webkit-transform: scale(-1, 1);
  -moz-transform: scale(-1, 1);
  -ms-transform: scale(-1, 1);
  -o-transform: scale(-1, 1);
  transform: scale(-1, 1);
}

.fa-flip-vertical {
  filter: progid:DXImageTransform.Microsoft.BasicImage(rotation=$rotation);
  -webkit-transform: scale(1, -1);
  -moz-transform: scale(1, -1);
  -ms-transform: scale(1, -1);
  -o-transform: scale(1, -1);
  transform: scale(1, -1);
}

.fa-stack {
  position: relative;
  display: inline-block;
  width: 2em;
  height: 2em;
  line-height: 2em;
  vertical-align: middle;
}

.fa-stack-1x, .fa-stack-2x {
  position: absolute;
  left: 0;
  width: 100%;
  text-align: center;
}

.fa-stack-1x {
  line-height: inherit;
}

.fa-stack-2x {
  font-size: 2em;
}

.fa-inverse {
  color: white;
}

/* Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
   readers do not read off random characters that represent icons */
.fa-glass:before {
  content: "\f000";
}

.fa-music:before {
  content: "\f001";
}

.fa-search:before, .icon-search:before {
  content: "\f002";
}

.fa-envelope-o:before {
  content: "\f003";
}

.fa-heart:before {
  content: "\f004";
}

.fa-star:before {
  content: "\f005";
}

.fa-star-o:before {
  content: "\f006";
}

.fa-user:before {
  content: "\f007";
}

.fa-film:before {
  content: "\f008";
}

.fa-th-large:before {
  content: "\f009";
}

.fa-th:before {
  content: "\f00a";
}

.fa-th-list:before {
  content: "\f00b";
}

.fa-check:before {
  content: "\f00c";
}

.fa-times:before, .wy-tag-input-group .wy-tag .wy-tag-remove:before {
  content: "\f00d";
}

.fa-search-plus:before {
  content: "\f00e";
}

.fa-search-minus:before {
  content: "\f010";
}

.fa-power-off:before {
  content: "\f011";
}

.fa-signal:before {
  content: "\f012";
}

.fa-gear:before,
.fa-cog:before {
  content: "\f013";
}

.fa-trash-o:before {
  content: "\f014";
}

.fa-home:before, .icon-home:before {
  content: "\f015";
}

.fa-file-o:before {
  content: "\f016";
}

.fa-clock-o:before {
  content: "\f017";
}

.fa-road:before {
  content: "\f018";
}

.fa-download:before {
  content: "\f019";
}

.fa-arrow-circle-o-down:before {
  content: "\f01a";
}

.fa-arrow-circle-o-up:before {
  content: "\f01b";
}

.fa-inbox:before {
  content: "\f01c";
}

.fa-play-circle-o:before {
  content: "\f01d";
}

.fa-rotate-right:before,
.fa-repeat:before {
  content: "\f01e";
}

.fa-refresh:before {
  content: "\f021";
}

.fa-list-alt:before {
  content: "\f022";
}

.fa-lock:before {
  content: "\f023";
}

.fa-flag:before {
  content: "\f024";
}

.fa-headphones:before {
  content: "\f025";
}

.fa-volume-off:before {
  content: "\f026";
}

.fa-volume-down:before {
  content: "\f027";
}

.fa-volume-up:before {
  content: "\f028";
}

.fa-qrcode:before {
  content: "\f029";
}

.fa-barcode:before {
  content: "\f02a";
}

.fa-tag:before {
  content: "\f02b";
}

.fa-tags:before {
  content: "\f02c";
}

.fa-book:before, .icon-book:before {
  content: "\f02d";
}

.fa-bookmark:before {
  content: "\f02e";
}

.fa-print:before {
  content: "\f02f";
}

.fa-camera:before {
  content: "\f030";
}

.fa-font:before {
  content: "\f031";
}

.fa-bold:before {
  content: "\f032";
}

.fa-italic:before {
  content: "\f033";
}

.fa-text-height:before {
  content: "\f034";
}

.fa-text-width:before {
  content: "\f035";
}

.fa-align-left:before {
  content: "\f036";
}

.fa-align-center:before {
  content: "\f037";
}

.fa-align-right:before {
  content: "\f038";
}

.fa-align-justify:before {
  content: "\f039";
}

.fa-list:before {
  content: "\f03a";
}

.fa-dedent:before,
.fa-outdent:before {
  content: "\f03b";
}

.fa-indent:before {
  content: "\f03c";
}

.fa-video-camera:before {
  content: "\f03d";
}

.fa-picture-o:before {
  content: "\f03e";
}

.fa-pencil:before {
  content: "\f040";
}

.fa-map-marker:before {
  content: "\f041";
}

.fa-adjust:before {
  content: "\f042";
}

.fa-tint:before {
  content: "\f043";
}

.fa-edit:before,
.fa-pencil-square-o:before {
  content: "\f044";
}

.fa-share-square-o:before {
  content: "\f045";
}

.fa-check-square-o:before {
  content: "\f046";
}

.fa-arrows:before {
  content: "\f047";
}

.fa-step-backward:before {
  content: "\f048";
}

.fa-fast-backward:before {
  content: "\f049";
}

.fa-backward:before {
  content: "\f04a";
}

.fa-play:before {
  content: "\f04b";
}

.fa-pause:before {
  content: "\f04c";
}

.fa-stop:before {
  content: "\f04d";
}

.fa-forward:before {
  content: "\f04e";
}

.fa-fast-forward:before {
  content: "\f050";
}

.fa-step-forward:before {
  content: "\f051";
}

.fa-eject:before {
  content: "\f052";
}

.fa-chevron-left:before {
  content: "\f053";
}

.fa-chevron-right:before {
  content: "\f054";
}

.fa-plus-circle:before {
  content: "\f055";
}

.fa-minus-circle:before {
  content: "\f056";
}

.fa-times-circle:before {
  content: "\f057";
}

.fa-check-circle:before, .wy-inline-validate.wy-inline-validate-success .wy-input-context:before {
  content: "\f058";
}

.fa-question-circle:before {
  content: "\f059";
}

.fa-info-circle:before {
  content: "\f05a";
}

.fa-crosshairs:before {
  content: "\f05b";
}

.fa-times-circle-o:before, .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before {
  content: "\f05c";
}

.fa-check-circle-o:before {
  content: "\f05d";
}

.fa-ban:before {
  content: "\f05e";
}

.fa-arrow-left:before {
  content: "\f060";
}

.fa-arrow-right:before {
  content: "\f061";
}

.fa-arrow-up:before {
  content: "\f062";
}

.fa-arrow-down:before {
  content: "\f063";
}

.fa-mail-forward:before,
.fa-share:before {
  content: "\f064";
}

.fa-expand:before {
  content: "\f065";
}

.fa-compress:before {
  content: "\f066";
}

.fa-plus:before {
  content: "\f067";
}

.fa-minus:before {
  content: "\f068";
}

.fa-asterisk:before {
  content: "\f069";
}

.fa-exclamation-circle:before, .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .rst-content .admonition-title:before {
  content: "\f06a";
}

.fa-gift:before {
  content: "\f06b";
}

.fa-leaf:before {
  content: "\f06c";
}

.fa-fire:before, .icon-fire:before {
  content: "\f06d";
}

.fa-eye:before {
  content: "\f06e";
}

.fa-eye-slash:before {
  content: "\f070";
}

.fa-warning:before,
.fa-exclamation-triangle:before {
  content: "\f071";
}

.fa-plane:before {
  content: "\f072";
}

.fa-calendar:before {
  content: "\f073";
}

.fa-random:before {
  content: "\f074";
}

.fa-comment:before {
  content: "\f075";
}

.fa-magnet:before {
  content: "\f076";
}

.fa-chevron-up:before {
  content: "\f077";
}

.fa-chevron-down:before {
  content: "\f078";
}

.fa-retweet:before {
  content: "\f079";
}

.fa-shopping-cart:before {
  content: "\f07a";
}

.fa-folder:before {
  content: "\f07b";
}

.fa-folder-open:before {
  content: "\f07c";
}

.fa-arrows-v:before {
  content: "\f07d";
}

.fa-arrows-h:before {
  content: "\f07e";
}

.fa-bar-chart-o:before {
  content: "\f080";
}

.fa-twitter-square:before {
  content: "\f081";
}

.fa-facebook-square:before {
  content: "\f082";
}

.fa-camera-retro:before {
  content: "\f083";
}

.fa-key:before {
  content: "\f084";
}

.fa-gears:before,
.fa-cogs:before {
  content: "\f085";
}

.fa-comments:before {
  content: "\f086";
}

.fa-thumbs-o-up:before {
  content: "\f087";
}

.fa-thumbs-o-down:before {
  content: "\f088";
}

.fa-star-half:before {
  content: "\f089";
}

.fa-heart-o:before {
  content: "\f08a";
}

.fa-sign-out:before {
  content: "\f08b";
}

.fa-linkedin-square:before {
  content: "\f08c";
}

.fa-thumb-tack:before {
  content: "\f08d";
}

.fa-external-link:before {
  content: "\f08e";
}

.fa-sign-in:before {
  content: "\f090";
}

.fa-trophy:before {
  content: "\f091";
}

.fa-github-square:before {
  content: "\f092";
}

.fa-upload:before {
  content: "\f093";
}

.fa-lemon-o:before {
  content: "\f094";
}

.fa-phone:before {
  content: "\f095";
}

.fa-square-o:before {
  content: "\f096";
}

.fa-bookmark-o:before {
  content: "\f097";
}

.fa-phone-square:before {
  content: "\f098";
}

.fa-twitter:before {
  content: "\f099";
}

.fa-facebook:before {
  content: "\f09a";
}

.fa-github:before, .icon-github:before {
  content: "\f09b";
}

.fa-unlock:before {
  content: "\f09c";
}

.fa-credit-card:before {
  content: "\f09d";
}

.fa-rss:before {
  content: "\f09e";
}

.fa-hdd-o:before {
  content: "\f0a0";
}

.fa-bullhorn:before {
  content: "\f0a1";
}

.fa-bell:before {
  content: "\f0f3";
}

.fa-certificate:before {
  content: "\f0a3";
}

.fa-hand-o-right:before {
  content: "\f0a4";
}

.fa-hand-o-left:before {
  content: "\f0a5";
}

.fa-hand-o-up:before {
  content: "\f0a6";
}

.fa-hand-o-down:before {
  content: "\f0a7";
}

.fa-arrow-circle-left:before, .icon-circle-arrow-left:before {
  content: "\f0a8";
}

.fa-arrow-circle-right:before, .icon-circle-arrow-right:before {
  content: "\f0a9";
}

.fa-arrow-circle-up:before {
  content: "\f0aa";
}

.fa-arrow-circle-down:before {
  content: "\f0ab";
}

.fa-globe:before {
  content: "\f0ac";
}

.fa-wrench:before {
  content: "\f0ad";
}

.fa-tasks:before {
  content: "\f0ae";
}

.fa-filter:before {
  content: "\f0b0";
}

.fa-briefcase:before {
  content: "\f0b1";
}

.fa-arrows-alt:before {
  content: "\f0b2";
}

.fa-group:before,
.fa-users:before {
  content: "\f0c0";
}

.fa-chain:before,
.fa-link:before,
.icon-link:before {
  content: "\f0c1";
}

.fa-cloud:before {
  content: "\f0c2";
}

.fa-flask:before {
  content: "\f0c3";
}

.fa-cut:before,
.fa-scissors:before {
  content: "\f0c4";
}

.fa-copy:before,
.fa-files-o:before {
  content: "\f0c5";
}

.fa-paperclip:before {
  content: "\f0c6";
}

.fa-save:before,
.fa-floppy-o:before {
  content: "\f0c7";
}

.fa-square:before {
  content: "\f0c8";
}

.fa-bars:before {
  content: "\f0c9";
}

.fa-list-ul:before {
  content: "\f0ca";
}

.fa-list-ol:before {
  content: "\f0cb";
}

.fa-strikethrough:before {
  content: "\f0cc";
}

.fa-underline:before {
  content: "\f0cd";
}

.fa-table:before {
  content: "\f0ce";
}

.fa-magic:before {
  content: "\f0d0";
}

.fa-truck:before {
  content: "\f0d1";
}

.fa-pinterest:before {
  content: "\f0d2";
}

.fa-pinterest-square:before {
  content: "\f0d3";
}

.fa-google-plus-square:before {
  content: "\f0d4";
}

.fa-google-plus:before {
  content: "\f0d5";
}

.fa-money:before {
  content: "\f0d6";
}

.fa-caret-down:before, .wy-dropdown .caret:before, .icon-caret-down:before {
  content: "\f0d7";
}

.fa-caret-up:before {
  content: "\f0d8";
}

.fa-caret-left:before {
  content: "\f0d9";
}

.fa-caret-right:before {
  content: "\f0da";
}

.fa-columns:before {
  content: "\f0db";
}

.fa-unsorted:before,
.fa-sort:before {
  content: "\f0dc";
}

.fa-sort-down:before,
.fa-sort-asc:before {
  content: "\f0dd";
}

.fa-sort-up:before,
.fa-sort-desc:before {
  content: "\f0de";
}

.fa-envelope:before {
  content: "\f0e0";
}

.fa-linkedin:before {
  content: "\f0e1";
}

.fa-rotate-left:before,
.fa-undo:before {
  content: "\f0e2";
}

.fa-legal:before,
.fa-gavel:before {
  content: "\f0e3";
}

.fa-dashboard:before,
.fa-tachometer:before {
  content: "\f0e4";
}

.fa-comment-o:before {
  content: "\f0e5";
}

.fa-comments-o:before {
  content: "\f0e6";
}

.fa-flash:before,
.fa-bolt:before {
  content: "\f0e7";
}

.fa-sitemap:before {
  content: "\f0e8";
}

.fa-umbrella:before {
  content: "\f0e9";
}

.fa-paste:before,
.fa-clipboard:before {
  content: "\f0ea";
}

.fa-lightbulb-o:before {
  content: "\f0eb";
}

.fa-exchange:before {
  content: "\f0ec";
}

.fa-cloud-download:before {
  content: "\f0ed";
}

.fa-cloud-upload:before {
  content: "\f0ee";
}

.fa-user-md:before {
  content: "\f0f0";
}

.fa-stethoscope:before {
  content: "\f0f1";
}

.fa-suitcase:before {
  content: "\f0f2";
}

.fa-bell-o:before {
  content: "\f0a2";
}

.fa-coffee:before {
  content: "\f0f4";
}

.fa-cutlery:before {
  content: "\f0f5";
}

.fa-file-text-o:before {
  content: "\f0f6";
}

.fa-building-o:before {
  content: "\f0f7";
}

.fa-hospital-o:before {
  content: "\f0f8";
}

.fa-ambulance:before {
  content: "\f0f9";
}

.fa-medkit:before {
  content: "\f0fa";
}

.fa-fighter-jet:before {
  content: "\f0fb";
}

.fa-beer:before {
  content: "\f0fc";
}

.fa-h-square:before {
  content: "\f0fd";
}

.fa-plus-square:before {
  content: "\f0fe";
}

.fa-angle-double-left:before {
  content: "\f100";
}

.fa-angle-double-right:before {
  content: "\f101";
}

.fa-angle-double-up:before {
  content: "\f102";
}

.fa-angle-double-down:before {
  content: "\f103";
}

.fa-angle-left:before {
  content: "\f104";
}

.fa-angle-right:before {
  content: "\f105";
}

.fa-angle-up:before {
  content: "\f106";
}

.fa-angle-down:before {
  content: "\f107";
}

.fa-desktop:before {
  content: "\f108";
}

.fa-laptop:before {
  content: "\f109";
}

.fa-tablet:before {
  content: "\f10a";
}

.fa-mobile-phone:before,
.fa-mobile:before {
  content: "\f10b";
}

.fa-circle-o:before {
  content: "\f10c";
}

.fa-quote-left:before {
  content: "\f10d";
}

.fa-quote-right:before {
  content: "\f10e";
}

.fa-spinner:before {
  content: "\f110";
}

.fa-circle:before {
  content: "\f111";
}

.fa-mail-reply:before,
.fa-reply:before {
  content: "\f112";
}

.fa-github-alt:before {
  content: "\f113";
}

.fa-folder-o:before {
  content: "\f114";
}

.fa-folder-open-o:before {
  content: "\f115";
}

.fa-smile-o:before {
  content: "\f118";
}

.fa-frown-o:before {
  content: "\f119";
}

.fa-meh-o:before {
  content: "\f11a";
}

.fa-gamepad:before {
  content: "\f11b";
}

.fa-keyboard-o:before {
  content: "\f11c";
}

.fa-flag-o:before {
  content: "\f11d";
}

.fa-flag-checkered:before {
  content: "\f11e";
}

.fa-terminal:before {
  content: "\f120";
}

.fa-code:before {
  content: "\f121";
}

.fa-reply-all:before {
  content: "\f122";
}

.fa-mail-reply-all:before {
  content: "\f122";
}

.fa-star-half-empty:before,
.fa-star-half-full:before,
.fa-star-half-o:before {
  content: "\f123";
}

.fa-location-arrow:before {
  content: "\f124";
}

.fa-crop:before {
  content: "\f125";
}

.fa-code-fork:before {
  content: "\f126";
}

.fa-unlink:before,
.fa-chain-broken:before {
  content: "\f127";
}

.fa-question:before {
  content: "\f128";
}

.fa-info:before {
  content: "\f129";
}

.fa-exclamation:before {
  content: "\f12a";
}

.fa-superscript:before {
  content: "\f12b";
}

.fa-subscript:before {
  content: "\f12c";
}

.fa-eraser:before {
  content: "\f12d";
}

.fa-puzzle-piece:before {
  content: "\f12e";
}

.fa-microphone:before {
  content: "\f130";
}

.fa-microphone-slash:before {
  content: "\f131";
}

.fa-shield:before {
  content: "\f132";
}

.fa-calendar-o:before {
  content: "\f133";
}

.fa-fire-extinguisher:before {
  content: "\f134";
}

.fa-rocket:before {
  content: "\f135";
}

.fa-maxcdn:before {
  content: "\f136";
}

.fa-chevron-circle-left:before {
  content: "\f137";
}

.fa-chevron-circle-right:before {
  content: "\f138";
}

.fa-chevron-circle-up:before {
  content: "\f139";
}

.fa-chevron-circle-down:before {
  content: "\f13a";
}

.fa-html5:before {
  content: "\f13b";
}

.fa-css3:before {
  content: "\f13c";
}

.fa-anchor:before {
  content: "\f13d";
}

.fa-unlock-alt:before {
  content: "\f13e";
}

.fa-bullseye:before {
  content: "\f140";
}

.fa-ellipsis-h:before {
  content: "\f141";
}

.fa-ellipsis-v:before {
  content: "\f142";
}

.fa-rss-square:before {
  content: "\f143";
}

.fa-play-circle:before {
  content: "\f144";
}

.fa-ticket:before {
  content: "\f145";
}

.fa-minus-square:before {
  content: "\f146";
}

.fa-minus-square-o:before {
  content: "\f147";
}

.fa-level-up:before {
  content: "\f148";
}

.fa-level-down:before {
  content: "\f149";
}

.fa-check-square:before {
  content: "\f14a";
}

.fa-pencil-square:before {
  content: "\f14b";
}

.fa-external-link-square:before {
  content: "\f14c";
}

.fa-share-square:before {
  content: "\f14d";
}

.fa-compass:before {
  content: "\f14e";
}

.fa-toggle-down:before,
.fa-caret-square-o-down:before {
  content: "\f150";
}

.fa-toggle-up:before,
.fa-caret-square-o-up:before {
  content: "\f151";
}

.fa-toggle-right:before,
.fa-caret-square-o-right:before {
  content: "\f152";
}

.fa-euro:before,
.fa-eur:before {
  content: "\f153";
}

.fa-gbp:before {
  content: "\f154";
}

.fa-dollar:before,
.fa-usd:before {
  content: "\f155";
}

.fa-rupee:before,
.fa-inr:before {
  content: "\f156";
}

.fa-cny:before,
.fa-rmb:before,
.fa-yen:before,
.fa-jpy:before {
  content: "\f157";
}

.fa-ruble:before,
.fa-rouble:before,
.fa-rub:before {
  content: "\f158";
}

.fa-won:before,
.fa-krw:before {
  content: "\f159";
}

.fa-bitcoin:before,
.fa-btc:before {
  content: "\f15a";
}

.fa-file:before {
  content: "\f15b";
}

.fa-file-text:before {
  content: "\f15c";
}

.fa-sort-alpha-asc:before {
  content: "\f15d";
}

.fa-sort-alpha-desc:before {
  content: "\f15e";
}

.fa-sort-amount-asc:before {
  content: "\f160";
}

.fa-sort-amount-desc:before {
  content: "\f161";
}

.fa-sort-numeric-asc:before {
  content: "\f162";
}

.fa-sort-numeric-desc:before {
  content: "\f163";
}

.fa-thumbs-up:before {
  content: "\f164";
}

.fa-thumbs-down:before {
  content: "\f165";
}

.fa-youtube-square:before {
  content: "\f166";
}

.fa-youtube:before {
  content: "\f167";
}

.fa-xing:before {
  content: "\f168";
}

.fa-xing-square:before {
  content: "\f169";
}

.fa-youtube-play:before {
  content: "\f16a";
}

.fa-dropbox:before {
  content: "\f16b";
}

.fa-stack-overflow:before {
  content: "\f16c";
}

.fa-instagram:before {
  content: "\f16d";
}

.fa-flickr:before {
  content: "\f16e";
}

.fa-adn:before {
  content: "\f170";
}

.fa-bitbucket:before, .icon-bitbucket:before {
  content: "\f171";
}

.fa-bitbucket-square:before {
  content: "\f172";
}

.fa-tumblr:before {
  content: "\f173";
}

.fa-tumblr-square:before {
  content: "\f174";
}

.fa-long-arrow-down:before {
  content: "\f175";
}

.fa-long-arrow-up:before {
  content: "\f176";
}

.fa-long-arrow-left:before {
  content: "\f177";
}

.fa-long-arrow-right:before {
  content: "\f178";
}

.fa-apple:before {
  content: "\f179";
}

.fa-windows:before {
  content: "\f17a";
}

.fa-android:before {
  content: "\f17b";
}

.fa-linux:before {
  content: "\f17c";
}

.fa-dribbble:before {
  content: "\f17d";
}

.fa-skype:before {
  content: "\f17e";
}

.fa-foursquare:before {
  content: "\f180";
}

.fa-trello:before {
  content: "\f181";
}

.fa-female:before {
  content: "\f182";
}

.fa-male:before {
  content: "\f183";
}

.fa-gittip:before {
  content: "\f184";
}

.fa-sun-o:before {
  content: "\f185";
}

.fa-moon-o:before {
  content: "\f186";
}

.fa-archive:before {
  content: "\f187";
}

.fa-bug:before {
  content: "\f188";
}

.fa-vk:before {
  content: "\f189";
}

.fa-weibo:before {
  content: "\f18a";
}

.fa-renren:before {
  content: "\f18b";
}

.fa-pagelines:before {
  content: "\f18c";
}

.fa-stack-exchange:before {
  content: "\f18d";
}

.fa-arrow-circle-o-right:before {
  content: "\f18e";
}

.fa-arrow-circle-o-left:before {
  content: "\f190";
}

.fa-toggle-left:before,
.fa-caret-square-o-left:before {
  content: "\f191";
}

.fa-dot-circle-o:before {
  content: "\f192";
}

.fa-wheelchair:before {
  content: "\f193";
}

.fa-vimeo-square:before {
  content: "\f194";
}

.fa-turkish-lira:before,
.fa-try:before {
  content: "\f195";
}

.fa-plus-square-o:before {
  content: "\f196";
}

.btn .fa, .btn .wy-dropdown .caret, .wy-dropdown .btn .caret, .btn .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .wy-input-context, .btn .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .btn .wy-tag-remove, .btn .rst-content .admonition-title, .rst-content .btn .admonition-title, .btn .rst-content h1 .headerlink, .rst-content h1 .btn .headerlink, .btn .rst-content h2 .headerlink, .rst-content h2 .btn .headerlink, .btn .rst-content h3 .headerlink, .rst-content h3 .btn .headerlink, .btn .rst-content h4 .headerlink, .rst-content h4 .btn .headerlink, .btn .rst-content h5 .headerlink, .rst-content h5 .btn .headerlink, .btn .rst-content h6 .headerlink, .rst-content h6 .btn .headerlink, .btn .rst-content dl dt .headerlink, .rst-content dl dt .btn .headerlink, .btn .icon, .nav .fa, .nav .wy-dropdown .caret, .wy-dropdown .nav .caret, .nav .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .wy-input-context, .nav .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .nav .wy-tag-remove, .nav .rst-content .admonition-title, .rst-content .nav .admonition-title, .nav .rst-content h1 .headerlink, .rst-content h1 .nav .headerlink, .nav .rst-content h2 .headerlink, .rst-content h2 .nav .headerlink, .nav .rst-content h3 .headerlink, .rst-content h3 .nav .headerlink, .nav .rst-content h4 .headerlink, .rst-content h4 .nav .headerlink, .nav .rst-content h5 .headerlink, .rst-content h5 .nav .headerlink, .nav .rst-content h6 .headerlink, .rst-content h6 .nav .headerlink, .nav .rst-content dl dt .headerlink, .rst-content dl dt .nav .headerlink, .nav .icon {
  display: inline;
}
.btn .fa.fa-large, .btn .wy-dropdown .fa-large.caret, .wy-dropdown .btn .fa-large.caret, .btn .wy-inline-validate.wy-inline-validate-success .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .fa-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .fa-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .fa-large.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .fa-large.wy-input-context, .btn .wy-tag-input-group .wy-tag .fa-large.wy-tag-remove, .wy-tag-input-group .wy-tag .btn .fa-large.wy-tag-remove, .btn .rst-content .fa-large.admonition-title, .rst-content .btn .fa-large.admonition-title, .btn .rst-content h1 .fa-large.headerlink, .rst-content h1 .btn .fa-large.headerlink, .btn .rst-content h2 .fa-large.headerlink, .rst-content h2 .btn .fa-large.headerlink, .btn .rst-content h3 .fa-large.headerlink, .rst-content h3 .btn .fa-large.headerlink, .btn .rst-content h4 .fa-large.headerlink, .rst-content h4 .btn .fa-large.headerlink, .btn .rst-content h5 .fa-large.headerlink, .rst-content h5 .btn .fa-large.headerlink, .btn .rst-content h6 .fa-large.headerlink, .rst-content h6 .btn .fa-large.headerlink, .btn .rst-content dl dt .fa-large.headerlink, .rst-content dl dt .btn .fa-large.headerlink, .btn .fa-large.icon, .nav .fa.fa-large, .nav .wy-dropdown .fa-large.caret, .wy-dropdown .nav .fa-large.caret, .nav .wy-inline-validate.wy-inline-validate-success .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .fa-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .fa-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .fa-large.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .fa-large.wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .fa-large.wy-input-context, .nav .wy-tag-input-group .wy-tag .fa-large.wy-tag-remove, .wy-tag-input-group .wy-tag .nav .fa-large.wy-tag-remove, .nav .rst-content .fa-large.admonition-title, .rst-content .nav .fa-large.admonition-title, .nav .rst-content h1 .fa-large.headerlink, .rst-content h1 .nav .fa-large.headerlink, .nav .rst-content h2 .fa-large.headerlink, .rst-content h2 .nav .fa-large.headerlink, .nav .rst-content h3 .fa-large.headerlink, .rst-content h3 .nav .fa-large.headerlink, .nav .rst-content h4 .fa-large.headerlink, .rst-content h4 .nav .fa-large.headerlink, .nav .rst-content h5 .fa-large.headerlink, .rst-content h5 .nav .fa-large.headerlink, .nav .rst-content h6 .fa-large.headerlink, .rst-content h6 .nav .fa-large.headerlink, .nav .rst-content dl dt .fa-large.headerlink, .rst-content dl dt .nav .fa-large.headerlink, .nav .fa-large.icon {
  line-height: 0.9em;
}
.btn .fa.fa-spin, .btn .wy-dropdown .fa-spin.caret, .wy-dropdown .btn .fa-spin.caret, .btn .wy-inline-validate.wy-inline-validate-success .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-success .btn .fa-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-danger .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .btn .fa-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-warning .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .btn .fa-spin.wy-input-context, .btn .wy-inline-validate.wy-inline-validate-info .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-info .btn .fa-spin.wy-input-context, .btn .wy-tag-input-group .wy-tag .fa-spin.wy-tag-remove, .wy-tag-input-group .wy-tag .btn .fa-spin.wy-tag-remove, .btn .rst-content .fa-spin.admonition-title, .rst-content .btn .fa-spin.admonition-title, .btn .rst-content h1 .fa-spin.headerlink, .rst-content h1 .btn .fa-spin.headerlink, .btn .rst-content h2 .fa-spin.headerlink, .rst-content h2 .btn .fa-spin.headerlink, .btn .rst-content h3 .fa-spin.headerlink, .rst-content h3 .btn .fa-spin.headerlink, .btn .rst-content h4 .fa-spin.headerlink, .rst-content h4 .btn .fa-spin.headerlink, .btn .rst-content h5 .fa-spin.headerlink, .rst-content h5 .btn .fa-spin.headerlink, .btn .rst-content h6 .fa-spin.headerlink, .rst-content h6 .btn .fa-spin.headerlink, .btn .rst-content dl dt .fa-spin.headerlink, .rst-content dl dt .btn .fa-spin.headerlink, .btn .fa-spin.icon, .nav .fa.fa-spin, .nav .wy-dropdown .fa-spin.caret, .wy-dropdown .nav .fa-spin.caret, .nav .wy-inline-validate.wy-inline-validate-success .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-success .nav .fa-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-danger .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-danger .nav .fa-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-warning .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-warning .nav .fa-spin.wy-input-context, .nav .wy-inline-validate.wy-inline-validate-info .fa-spin.wy-input-context, .wy-inline-validate.wy-inline-validate-info .nav .fa-spin.wy-input-context, .nav .wy-tag-input-group .wy-tag .fa-spin.wy-tag-remove, .wy-tag-input-group .wy-tag .nav .fa-spin.wy-tag-remove, .nav .rst-content .fa-spin.admonition-title, .rst-content .nav .fa-spin.admonition-title, .nav .rst-content h1 .fa-spin.headerlink, .rst-content h1 .nav .fa-spin.headerlink, .nav .rst-content h2 .fa-spin.headerlink, .rst-content h2 .nav .fa-spin.headerlink, .nav .rst-content h3 .fa-spin.headerlink, .rst-content h3 .nav .fa-spin.headerlink, .nav .rst-content h4 .fa-spin.headerlink, .rst-content h4 .nav .fa-spin.headerlink, .nav .rst-content h5 .fa-spin.headerlink, .rst-content h5 .nav .fa-spin.headerlink, .nav .rst-content h6 .fa-spin.headerlink, .rst-content h6 .nav .fa-spin.headerlink, .nav .rst-content dl dt .fa-spin.headerlink, .rst-content dl dt .nav .fa-spin.headerlink, .nav .fa-spin.icon {
  display: inline-block;
}

.btn.fa:before, .wy-dropdown .btn.caret:before, .wy-inline-validate.wy-inline-validate-success .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .btn.wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .btn.wy-input-context:before, .wy-tag-input-group .wy-tag .btn.wy-tag-remove:before, .rst-content .btn.admonition-title:before, .rst-content h1 .btn.headerlink:before, .rst-content h2 .btn.headerlink:before, .rst-content h3 .btn.headerlink:before, .rst-content h4 .btn.headerlink:before, .rst-content h5 .btn.headerlink:before, .rst-content h6 .btn.headerlink:before, .rst-content dl dt .btn.headerlink:before, .btn.icon:before {
  opacity: 0.5;
  -webkit-transition: opacity 0.05s ease-in;
  -moz-transition: opacity 0.05s ease-in;
  transition: opacity 0.05s ease-in;
}

.btn.fa:hover:before, .wy-dropdown .btn.caret:hover:before, .wy-inline-validate.wy-inline-validate-success .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-danger .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-warning .btn.wy-input-context:hover:before, .wy-inline-validate.wy-inline-validate-info .btn.wy-input-context:hover:before, .wy-tag-input-group .wy-tag .btn.wy-tag-remove:hover:before, .rst-content .btn.admonition-title:hover:before, .rst-content h1 .btn.headerlink:hover:before, .rst-content h2 .btn.headerlink:hover:before, .rst-content h3 .btn.headerlink:hover:before, .rst-content h4 .btn.headerlink:hover:before, .rst-content h5 .btn.headerlink:hover:before, .rst-content h6 .btn.headerlink:hover:before, .rst-content dl dt .btn.headerlink:hover:before, .btn.icon:hover:before {
  opacity: 1;
}

.btn-mini .fa:before, .btn-mini .wy-dropdown .caret:before, .wy-dropdown .btn-mini .caret:before, .btn-mini .wy-inline-validate.wy-inline-validate-success .wy-input-context:before, .wy-inline-validate.wy-inline-validate-success .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-danger .wy-input-context:before, .wy-inline-validate.wy-inline-validate-danger .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-warning .wy-input-context:before, .wy-inline-validate.wy-inline-validate-warning .btn-mini .wy-input-context:before, .btn-mini .wy-inline-validate.wy-inline-validate-info .wy-input-context:before, .wy-inline-validate.wy-inline-validate-info .btn-mini .wy-input-context:before, .btn-mini .wy-tag-input-group .wy-tag .wy-tag-remove:before, .wy-tag-input-group .wy-tag .btn-mini .wy-tag-remove:before, .btn-mini .rst-content .admonition-title:before, .rst-content .btn-mini .admonition-title:before, .btn-mini .rst-content h1 .headerlink:before, .rst-content h1 .btn-mini .headerlink:before, .btn-mini .rst-content h2 .headerlink:before, .rst-content h2 .btn-mini .headerlink:before, .btn-mini .rst-content h3 .headerlink:before, .rst-content h3 .btn-mini .headerlink:before, .btn-mini .rst-content h4 .headerlink:before, .rst-content h4 .btn-mini .headerlink:before, .btn-mini .rst-content h5 .headerlink:before, .rst-content h5 .btn-mini .headerlink:before, .btn-mini .rst-content h6 .headerlink:before, .rst-content h6 .btn-mini .headerlink:before, .btn-mini .rst-content dl dt .headerlink:before, .rst-content dl dt .btn-mini .headerlink:before, .btn-mini .icon:before {
  font-size: 14px;
  vertical-align: -15%;
}

.wy-alert, .rst-content .note, .rst-content .attention, .rst-content .caution, .rst-content .danger, .rst-content .error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .warning, .rst-content .seealso {
  padding: 12px;
  line-height: 24px;
  margin-bottom: 24px;
}

.wy-alert-title, .rst-content .admonition-title {
  color: white;
  font-weight: bold;
  display: block;
  color: white;
  background: transparent;
  margin: -12px;
  padding: 6px 12px;
  margin-bottom: 12px;
}

.wy-alert.wy-alert-danger, .rst-content .wy-alert-danger.note, .rst-content .wy-alert-danger.attention, .rst-content .wy-alert-danger.caution, .rst-content .danger, .rst-content .error, .rst-content .wy-alert-danger.hint, .rst-content .wy-alert-danger.important, .rst-content .wy-alert-danger.tip, .rst-content .wy-alert-danger.warning, .rst-content .wy-alert-danger.seealso {
  background: #fdf3f2;
}
.wy-alert.wy-alert-danger .wy-alert-title, .rst-content .wy-alert-danger.note .wy-alert-title, .rst-content .wy-alert-danger.attention .wy-alert-title, .rst-content .wy-alert-danger.caution .wy-alert-title, .rst-content .danger .wy-alert-title, .rst-content .error .wy-alert-title, .rst-content .wy-alert-danger.hint .wy-alert-title, .rst-content .wy-alert-danger.important .wy-alert-title, .rst-content .wy-alert-danger.tip .wy-alert-title, .rst-content .wy-alert-danger.warning .wy-alert-title, .rst-content .wy-alert-danger.seealso .wy-alert-title, .wy-alert.wy-alert-danger .rst-content .admonition-title, .rst-content .wy-alert.wy-alert-danger .admonition-title, .rst-content .wy-alert-danger.note .admonition-title, .rst-content .wy-alert-danger.attention .admonition-title, .rst-content .wy-alert-danger.caution .admonition-title, .rst-content .danger .admonition-title, .rst-content .error .admonition-title, .rst-content .wy-alert-danger.hint .admonition-title, .rst-content .wy-alert-danger.important .admonition-title, .rst-content .wy-alert-danger.tip .admonition-title, .rst-content .wy-alert-danger.warning .admonition-title, .rst-content .wy-alert-danger.seealso .admonition-title {
  background: #f29f97;
}

.wy-alert.wy-alert-warning, .rst-content .wy-alert-warning.note, .rst-content .attention, .rst-content .caution, .rst-content .wy-alert-warning.danger, .rst-content .wy-alert-warning.error, .rst-content .wy-alert-warning.hint, .rst-content .wy-alert-warning.important, .rst-content .wy-alert-warning.tip, .rst-content .warning, .rst-content .wy-alert-warning.seealso {
  background: #ffedcc;
}
.wy-alert.wy-alert-warning .wy-alert-title, .rst-content .wy-alert-warning.note .wy-alert-title, .rst-content .attention .wy-alert-title, .rst-content .caution .wy-alert-title, .rst-content .wy-alert-warning.danger .wy-alert-title, .rst-content .wy-alert-warning.error .wy-alert-title, .rst-content .wy-alert-warning.hint .wy-alert-title, .rst-content .wy-alert-warning.important .wy-alert-title, .rst-content .wy-alert-warning.tip .wy-alert-title, .rst-content .warning .wy-alert-title, .rst-content .wy-alert-warning.seealso .wy-alert-title, .wy-alert.wy-alert-warning .rst-content .admonition-title, .rst-content .wy-alert.wy-alert-warning .admonition-title, .rst-content .wy-alert-warning.note .admonition-title, .rst-content .attention .admonition-title, .rst-content .caution .admonition-title, .rst-content .wy-alert-warning.danger .admonition-title, .rst-content .wy-alert-warning.error .admonition-title, .rst-content .wy-alert-warning.hint .admonition-title, .rst-content .wy-alert-warning.important .admonition-title, .rst-content .wy-alert-warning.tip .admonition-title, .rst-content .warning .admonition-title, .rst-content .wy-alert-warning.seealso .admonition-title {
  background: #f0b37e;
}

.wy-alert.wy-alert-info, .rst-content .note, .rst-content .wy-alert-info.attention, .rst-content .wy-alert-info.caution, .rst-content .wy-alert-info.danger, .rst-content .wy-alert-info.error, .rst-content .wy-alert-info.hint, .rst-content .wy-alert-info.important, .rst-content .wy-alert-info.tip, .rst-content .wy-alert-info.warning, .rst-content .seealso {
  background: #e7f2fa;
}
.wy-alert.wy-alert-info .wy-alert-title, .rst-content .note .wy-alert-title, .rst-content .wy-alert-info.attention .wy-alert-title, .rst-content .wy-alert-info.caution .wy-alert-title, .rst-content .wy-alert-info.danger .wy-alert-title, .rst-content .wy-alert-info.error .wy-alert-title, .rst-content .wy-alert-info.hint .wy-alert-title, .rst-content .wy-alert-info.important .wy-alert-title, .rst-content .wy-alert-info.tip .wy-alert-title, .rst-content .wy-alert-info.warning .wy-alert-title, .rst-content .seealso .wy-alert-title, .wy-alert.wy-alert-info .rst-content .admonition-title, .rst-content .wy-alert.wy-alert-info .admonition-title, .rst-content .note .admonition-title, .rst-content .wy-alert-info.attention .admonition-title, .rst-content .wy-alert-info.caution .admonition-title, .rst-content .wy-alert-info.danger .admonition-title, .rst-content .wy-alert-info.error .admonition-title, .rst-content .wy-alert-info.hint .admonition-title, .rst-content .wy-alert-info.important .admonition-title, .rst-content .wy-alert-info.tip .admonition-title, .rst-content .wy-alert-info.warning .admonition-title, .rst-content .seealso .admonition-title {
  background: #6ab0de;
}

.wy-alert.wy-alert-success, .rst-content .wy-alert-success.note, .rst-content .wy-alert-success.attention, .rst-content .wy-alert-success.caution, .rst-content .wy-alert-success.danger, .rst-content .wy-alert-success.error, .rst-content .hint, .rst-content .important, .rst-content .tip, .rst-content .wy-alert-success.warning, .rst-content .wy-alert-success.seealso {
  background: #dbfaf4;
}
.wy-alert.wy-alert-success .wy-alert-title, .rst-content .wy-alert-success.note .wy-alert-title, .rst-content .wy-alert-success.attention .wy-alert-title, .rst-content .wy-alert-success.caution .wy-alert-title, .rst-content .wy-alert-success.danger .wy-alert-title, .rst-content .wy-alert-success.error .wy-alert-title, .rst-content .hint .wy-alert-title, .rst-content .important .wy-alert-title, .rst-content .tip .wy-alert-title, .rst-content .wy-alert-success.warning .wy-alert-title, .rst-content .wy-alert-success.seealso .wy-alert-title, .wy-alert.wy-alert-success .rst-content .admonition-title, .rst-content .wy-alert.wy-alert-success .admonition-title, .rst-content .wy-alert-success.note .admonition-title, .rst-content .wy-alert-success.attention .admonition-title, .rst-content .wy-alert-success.caution .admonition-title, .rst-content .wy-alert-success.danger .admonition-title, .rst-content .wy-alert-success.error .admonition-title, .rst-content .hint .admonition-title, .rst-content .important .admonition-title, .rst-content .tip .admonition-title, .rst-content .wy-alert-success.warning .admonition-title, .rst-content .wy-alert-success.seealso .admonition-title {
  background: #1abc9c;
}

.wy-alert.wy-alert-neutral, .rst-content .wy-alert-neutral.note, .rst-content .wy-alert-neutral.attention, .rst-content .wy-alert-neutral.caution, .rst-content .wy-alert-neutral.danger, .rst-content .wy-alert-neutral.error, .rst-content .wy-alert-neutral.hint, .rst-content .wy-alert-neutral.important, .rst-content .wy-alert-neutral.tip, .rst-content .wy-alert-neutral.warning, .rst-content .wy-alert-neutral.seealso {
  background: #f3f6f6;
}
.wy-alert.wy-alert-neutral strong, .rst-content .wy-alert-neutral.note strong, .rst-content .wy-alert-neutral.attention strong, .rst-content .wy-alert-neutral.caution strong, .rst-content .wy-alert-neutral.danger strong, .rst-content .wy-alert-neutral.error strong, .rst-content .wy-alert-neutral.hint strong, .rst-content .wy-alert-neutral.important strong, .rst-content .wy-alert-neutral.tip strong, .rst-content .wy-alert-neutral.warning strong, .rst-content .wy-alert-neutral.seealso strong {
  color: #404040;
}
.wy-alert.wy-alert-neutral a, .rst-content .wy-alert-neutral.note a, .rst-content .wy-alert-neutral.attention a, .rst-content .wy-alert-neutral.caution a, .rst-content .wy-alert-neutral.danger a, .rst-content .wy-alert-neutral.error a, .rst-content .wy-alert-neutral.hint a, .rst-content .wy-alert-neutral.important a, .rst-content .wy-alert-neutral.tip a, .rst-content .wy-alert-neutral.warning a, .rst-content .wy-alert-neutral.seealso a {
  color: #2980b9;
}

.wy-tray-container {
  position: fixed;
  top: -50px;
  left: 0;
  width: 100%;
  -webkit-transition: top 0.2s ease-in;
  -moz-transition: top 0.2s ease-in;
  transition: top 0.2s ease-in;
}
.wy-tray-container.on {
  top: 0;
}
.wy-tray-container li {
  display: none;
  width: 100%;
  background: #343131;
  padding: 12px 24px;
  color: white;
  margin-bottom: 6px;
  text-align: center;
  box-shadow: 0 5px 5px 0 rgba(0, 0, 0, 0.1), 0px -1px 2px -1px rgba(255, 255, 255, 0.5) inset;
}
.wy-tray-container li.wy-tray-item-success {
  background: #27ae60;
}
.wy-tray-container li.wy-tray-item-info {
  background: #2980b9;
}
.wy-tray-container li.wy-tray-item-warning {
  background: #e67e22;
}
.wy-tray-container li.wy-tray-item-danger {
  background: #e74c3c;
}

.btn {
  /* Structure */
  display: inline-block;
  *display: inline;
  /*IE 6/7 */
  zoom: 1;
  line-height: normal;
  white-space: nowrap;
  vertical-align: baseline;
  text-align: center;
  cursor: pointer;
  -webkit-user-drag: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  font-size: 100%;
  padding: 6px 12px;
  color: white;
  border: 1px solid rgba(0, 0, 0, 0.1);
  border-bottom: solid 3px rgba(0, 0, 0, 0.1);
  background-color: #27ae60;
  text-decoration: none;
  font-weight: 500;
  box-shadow: 0px 1px 2px -1px rgba(255, 255, 255, 0.5) inset;
  -webkit-transition: all 0.1s linear;
  -moz-transition: all 0.1s linear;
  transition: all 0.1s linear;
  outline-none: false;
}

.btn-hover {
  background: #2e8ece;
  color: white;
}

.btn:hover {
  background: #2cc36b;
  color: white;
}
.btn:focus {
  background: #2cc36b;
  color: white;
  outline: 0;
}
.btn:active {
  border-top: solid 3px rgba(0, 0, 0, 0.1);
  border-bottom: solid 1px rgba(0, 0, 0, 0.1);
  box-shadow: 0px 1px 2px -1px rgba(0, 0, 0, 0.5) inset;
}
.btn[disabled] {
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  filter: alpha(opacity=40);
  opacity: 0.4;
  cursor: not-allowed;
  box-shadow: none;
}
.btn:visited {
  color: white;
}

.btn-disabled {
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  filter: alpha(opacity=40);
  opacity: 0.4;
  cursor: not-allowed;
  box-shadow: none;
}
.btn-disabled:hover, .btn-disabled:focus, .btn-disabled:active {
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  filter: alpha(opacity=40);
  opacity: 0.4;
  cursor: not-allowed;
  box-shadow: none;
}

/* Firefox: Get rid of the inner focus border */
.btn::-moz-focus-inner {
  padding: 0;
  border: 0;
}

.btn-small {
  font-size: 80%;
}

.btn-info {
  background-color: #2980b9 !important;
}
.btn-info:hover {
  background-color: #2e8ece !important;
}

.btn-neutral {
  background-color: #f3f6f6 !important;
  color: #404040 !important;
}
.btn-neutral:hover {
  background-color: #e5ebeb !important;
  color: #404040;
}
.btn-neutral:visited {
  color: #404040 !important;
}

.btn-success {
  background-color: #27ae60 !important;
}
.btn-success:hover {
  background-color: #229955 !important;
}

.btn-danger {
  background-color: #e74c3c !important;
}
.btn-danger:hover {
  background-color: #ea6153 !important;
}

.btn-warning {
  background-color: #e67e22 !important;
}
.btn-warning:hover {
  background-color: #e98b39 !important;
}

.btn-invert {
  background-color: #343131;
}
.btn-invert:hover {
  background-color: #413d3d !important;
}

.btn-link {
  background-color: transparent !important;
  color: #2980b9;
  border-color: transparent;
  box-shadow: none;
}
.btn-link:hover {
  background-color: transparent !important;
  color: #409ad5;
  border-color: transparent;
  box-shadow: none;
}
.btn-link:active {
  background-color: transparent !important;
  border-color: transparent;
  border-top: solid 1px transparent;
  border-bottom: solid 3px transparent;
  box-shadow: none;
}
.btn-link:visited {
  color: #9b59b6;
}

.wy-btn-group .btn, .wy-control .btn {
  vertical-align: middle;
}

.wy-btn-group {
  margin-bottom: 24px;
  *zoom: 1;
}
.wy-btn-group:before, .wy-btn-group:after {
  display: table;
  content: "";
}
.wy-btn-group:after {
  clear: both;
}

.wy-dropdown {
  position: relative;
  display: inline-block;
}

.wy-dropdown-menu {
  position: absolute;
  top: 100%;
  left: 0;
  display: none;
  float: left;
  min-width: 100%;
  background: #fcfcfc;
  z-index: 100;
  border: solid 1px #cfd7dd;
  box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.1);
  padding: 12px;
}
.wy-dropdown-menu > dd > a {
  display: block;
  clear: both;
  color: #404040;
  white-space: nowrap;
  font-size: 90%;
  padding: 0 12px;
}
.wy-dropdown-menu > dd > a:hover {
  background: #2980b9;
  color: white;
}
.wy-dropdown-menu > dd.divider {
  border-top: solid 1px #cfd7dd;
  margin: 6px 0;
}
.wy-dropdown-menu > dd.search {
  padding-bottom: 12px;
}
.wy-dropdown-menu > dd.search input[type="search"] {
  width: 100%;
}
.wy-dropdown-menu > dd.call-to-action {
  background: #e3e3e3;
  text-transform: uppercase;
  font-weight: 500;
  font-size: 80%;
}
.wy-dropdown-menu > dd.call-to-action:hover {
  background: #e3e3e3;
}
.wy-dropdown-menu > dd.call-to-action .btn {
  color: white;
}

.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu {
  background: #fcfcfc;
  margin-top: 2px;
}
.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu a {
  padding: 6px 12px;
}
.wy-dropdown.wy-dropdown-bubble .wy-dropdown-menu a:hover {
  background: #2980b9;
  color: white;
}

.wy-dropdown.wy-dropdown-left .wy-dropdown-menu {
  right: 0;
  text-align: right;
}

.wy-dropdown-arrow:before {
  content: " ";
  border-bottom: 5px solid whitesmoke;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  position: absolute;
  display: block;
  top: -4px;
  left: 50%;
  margin-left: -3px;
}
.wy-dropdown-arrow.wy-dropdown-arrow-left:before {
  left: 11px;
}

.wy-form-stacked select {
  display: block;
}

.wy-form-aligned input, .wy-form-aligned textarea, .wy-form-aligned select, .wy-form-aligned .wy-help-inline, .wy-form-aligned label {
  display: inline-block;
  *display: inline;
  *zoom: 1;
  vertical-align: middle;
}

.wy-form-aligned .wy-control-group > label {
  display: inline-block;
  vertical-align: middle;
  width: 10em;
  margin: 0.5em 1em 0 0;
  float: left;
}
.wy-form-aligned .wy-control {
  float: left;
}
.wy-form-aligned .wy-control label {
  display: block;
}
.wy-form-aligned .wy-control select {
  margin-top: 0.5em;
}

fieldset {
  border: 0;
  margin: 0;
  padding: 0;
}

legend {
  display: block;
  width: 100%;
  border: 0;
  padding: 0;
  white-space: normal;
  margin-bottom: 24px;
  font-size: 150%;
  *margin-left: -7px;
}

label {
  display: block;
  margin: 0 0 0.3125em 0;
  color: #999999;
  font-size: 90%;
}

button, input, select, textarea {
  font-size: 100%;
  margin: 0;
  vertical-align: baseline;
  *vertical-align: middle;
}

button, input {
  line-height: normal;
}

button {
  -webkit-appearance: button;
  cursor: pointer;
  *overflow: visible;
}

button::-moz-focus-inner, input::-moz-focus-inner {
  border: 0;
  padding: 0;
}

button[disabled] {
  cursor: default;
}

input[type="button"], input[type="reset"], input[type="submit"] {
  -webkit-appearance: button;
  cursor: pointer;
  *overflow: visible;
}
input[type="text"], input[type="password"], input[type="email"], input[type="url"], input[type="date"], input[type="month"], input[type="time"], input[type="datetime"], input[type="datetime-local"], input[type="week"], input[type="number"], input[type="search"], input[type="tel"], input[type="color"] {
  -webkit-appearance: none;
  padding: 6px;
  display: inline-block;
  border: 1px solid #cccccc;
  font-size: 80%;
  font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
  box-shadow: inset 0 1px 3px #dddddd;
  border-radius: 0;
  -webkit-transition: border 0.3s linear;
  -moz-transition: border 0.3s linear;
  transition: border 0.3s linear;
}
input[type="datetime-local"] {
  padding: 0.34375em 0.625em;
}
input[disabled] {
  cursor: default;
}
input[type="checkbox"], input[type="radio"] {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
  padding: 0;
  margin-right: 0.3125em;
  *height: 13px;
  *width: 13px;
}
input[type="search"] {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}
input[type="search"]::-webkit-search-cancel-button, input[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none;
}
input[type="text"]:focus, input[type="password"]:focus, input[type="email"]:focus, input[type="url"]:focus, input[type="date"]:focus, input[type="month"]:focus, input[type="time"]:focus, input[type="datetime"]:focus, input[type="datetime-local"]:focus, input[type="week"]:focus, input[type="number"]:focus, input[type="search"]:focus, input[type="tel"]:focus, input[type="color"]:focus {
  outline: 0;
  outline: thin dotted \9;
  border-color: #333333;
}
input.no-focus:focus {
  border-color: #cccccc !important;
}
input[type="file"]:focus, input[type="radio"]:focus, input[type="checkbox"]:focus {
  outline: thin dotted #333333;
  outline: 1px auto #129fea;
}
input[type="text"][disabled], input[type="password"][disabled], input[type="email"][disabled], input[type="url"][disabled], input[type="date"][disabled], input[type="month"][disabled], input[type="time"][disabled], input[type="datetime"][disabled], input[type="datetime-local"][disabled], input[type="week"][disabled], input[type="number"][disabled], input[type="search"][disabled], input[type="tel"][disabled], input[type="color"][disabled] {
  cursor: not-allowed;
  background-color: #f3f6f6;
  color: #cad2d3;
}

input:focus:invalid, textarea:focus:invalid, select:focus:invalid {
  color: #e74c3c;
  border: 1px solid #e74c3c;
}

input:focus:invalid:focus, textarea:focus:invalid:focus, select:focus:invalid:focus {
  border-color: #e74c3c;
}

input[type="file"]:focus:invalid:focus, input[type="radio"]:focus:invalid:focus, input[type="checkbox"]:focus:invalid:focus {
  outline-color: #e74c3c;
}

input.wy-input-large {
  padding: 12px;
  font-size: 100%;
}

textarea {
  overflow: auto;
  vertical-align: top;
  width: 100%;
}

select, textarea {
  padding: 0.5em 0.625em;
  display: inline-block;
  border: 1px solid #cccccc;
  font-size: 0.8em;
  box-shadow: inset 0 1px 3px #dddddd;
  -webkit-transition: border 0.3s linear;
  -moz-transition: border 0.3s linear;
  transition: border 0.3s linear;
}

select {
  border: 1px solid #cccccc;
  background-color: white;
}
select[multiple] {
  height: auto;
}

select:focus, textarea:focus {
  outline: 0;
}

select[disabled], textarea[disabled], input[readonly], select[readonly], textarea[readonly] {
  cursor: not-allowed;
  background-color: white;
  color: #cad2d3;
  border-color: transparent;
}

.wy-checkbox, .wy-radio {
  margin: 0.5em 0;
  color: #404040;
  display: block;
}

.wy-form-message-inline {
  display: inline-block;
  *display: inline;
  *zoom: 1;
  vertical-align: middle;
}

.wy-input-prefix, .wy-input-suffix {
  white-space: nowrap;
}
.wy-input-prefix .wy-input-context, .wy-input-suffix .wy-input-context {
  padding: 6px;
  display: inline-block;
  font-size: 80%;
  background-color: #f3f6f6;
  border: solid 1px #cccccc;
  color: #999999;
}

.wy-input-suffix .wy-input-context {
  border-left: 0;
}

.wy-input-prefix .wy-input-context {
  border-right: 0;
}

.wy-inline-validate {
  white-space: nowrap;
}
.wy-inline-validate .wy-input-context {
  padding: 0.5em 0.625em;
  display: inline-block;
  font-size: 80%;
}

.wy-inline-validate.wy-inline-validate-success .wy-input-context {
  color: #27ae60;
}

.wy-inline-validate.wy-inline-validate-danger .wy-input-context {
  color: #e74c3c;
}

.wy-inline-validate.wy-inline-validate-warning .wy-input-context {
  color: #e67e22;
}

.wy-inline-validate.wy-inline-validate-info .wy-input-context {
  color: #2980b9;
}

.wy-control-group {
  margin-bottom: 24px;
  *zoom: 1;
  max-width: 68em;
  margin-left: auto;
  margin-right: auto;
  *zoom: 1;
}
.wy-control-group:before, .wy-control-group:after {
  display: table;
  content: "";
}
.wy-control-group:after {
  clear: both;
}
.wy-control-group:before, .wy-control-group:after {
  display: table;
  content: "";
}
.wy-control-group:after {
  clear: both;
}

.wy-control-group.wy-control-group-error .wy-form-message, .wy-control-group.wy-control-group-error > label {
  color: #e74c3c;
}
.wy-control-group.wy-control-group-error input[type="text"], .wy-control-group.wy-control-group-error input[type="password"], .wy-control-group.wy-control-group-error input[type="email"], .wy-control-group.wy-control-group-error input[type="url"], .wy-control-group.wy-control-group-error input[type="date"], .wy-control-group.wy-control-group-error input[type="month"], .wy-control-group.wy-control-group-error input[type="time"], .wy-control-group.wy-control-group-error input[type="datetime"], .wy-control-group.wy-control-group-error input[type="datetime-local"], .wy-control-group.wy-control-group-error input[type="week"], .wy-control-group.wy-control-group-error input[type="number"], .wy-control-group.wy-control-group-error input[type="search"], .wy-control-group.wy-control-group-error input[type="tel"], .wy-control-group.wy-control-group-error input[type="color"] {
  border: solid 1px #e74c3c;
}
.wy-control-group.wy-control-group-error textarea {
  border: solid 1px #e74c3c;
}

.wy-control-group.wy-control-group-required > label:after {
  content: " *";
  color: #e74c3c;
}

.wy-control-group .wy-form-full, .wy-control-group .wy-form-halves, .wy-control-group .wy-form-thirds {
  padding-bottom: 12px;
}
.wy-control-group .wy-form-full select, .wy-control-group .wy-form-halves select, .wy-control-group .wy-form-thirds select {
  width: 100%;
}
.wy-control-group .wy-form-full input[type="text"], .wy-control-group .wy-form-full input[type="password"], .wy-control-group .wy-form-full input[type="email"], .wy-control-group .wy-form-full input[type="url"], .wy-control-group .wy-form-full input[type="date"], .wy-control-group .wy-form-full input[type="month"], .wy-control-group .wy-form-full input[type="time"], .wy-control-group .wy-form-full input[type="datetime"], .wy-control-group .wy-form-full input[type="datetime-local"], .wy-control-group .wy-form-full input[type="week"], .wy-control-group .wy-form-full input[type="number"], .wy-control-group .wy-form-full input[type="search"], .wy-control-group .wy-form-full input[type="tel"], .wy-control-group .wy-form-full input[type="color"], .wy-control-group .wy-form-halves input[type="text"], .wy-control-group .wy-form-halves input[type="password"], .wy-control-group .wy-form-halves input[type="email"], .wy-control-group .wy-form-halves input[type="url"], .wy-control-group .wy-form-halves input[type="date"], .wy-control-group .wy-form-halves input[type="month"], .wy-control-group .wy-form-halves input[type="time"], .wy-control-group .wy-form-halves input[type="datetime"], .wy-control-group .wy-form-halves input[type="datetime-local"], .wy-control-group .wy-form-halves input[type="week"], .wy-control-group .wy-form-halves input[type="number"], .wy-control-group .wy-form-halves input[type="search"], .wy-control-group .wy-form-halves input[type="tel"], .wy-control-group .wy-form-halves input[type="color"], .wy-control-group .wy-form-thirds input[type="text"], .wy-control-group .wy-form-thirds input[type="password"], .wy-control-group .wy-form-thirds input[type="email"], .wy-control-group .wy-form-thirds input[type="url"], .wy-control-group .wy-form-thirds input[type="date"], .wy-control-group .wy-form-thirds input[type="month"], .wy-control-group .wy-form-thirds input[type="time"], .wy-control-group .wy-form-thirds input[type="datetime"], .wy-control-group .wy-form-thirds input[type="datetime-local"], .wy-control-group .wy-form-thirds input[type="week"], .wy-control-group .wy-form-thirds input[type="number"], .wy-control-group .wy-form-thirds input[type="search"], .wy-control-group .wy-form-thirds input[type="tel"], .wy-control-group .wy-form-thirds input[type="color"] {
  width: 100%;
}

.wy-control-group .wy-form-full {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 100%;
  margin-right: 0;
}
.wy-control-group .wy-form-full:last-child {
  margin-right: 0;
}

.wy-control-group .wy-form-halves {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 48.82117%;
}
.wy-control-group .wy-form-halves:last-child {
  margin-right: 0;
}
.wy-control-group .wy-form-halves:nth-of-type(2n) {
  margin-right: 0;
}
.wy-control-group .wy-form-halves:nth-of-type(2n+1) {
  clear: left;
}

.wy-control-group .wy-form-thirds {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 31.76157%;
}
.wy-control-group .wy-form-thirds:last-child {
  margin-right: 0;
}
.wy-control-group .wy-form-thirds:nth-of-type(3n) {
  margin-right: 0;
}
.wy-control-group .wy-form-thirds:nth-of-type(3n+1) {
  clear: left;
}

.wy-control-group.wy-control-group-no-input .wy-control {
  margin: 0.5em 0 0 0;
  font-size: 90%;
}

.wy-control-group.fluid-input input[type="text"], .wy-control-group.fluid-input input[type="password"], .wy-control-group.fluid-input input[type="email"], .wy-control-group.fluid-input input[type="url"], .wy-control-group.fluid-input input[type="date"], .wy-control-group.fluid-input input[type="month"], .wy-control-group.fluid-input input[type="time"], .wy-control-group.fluid-input input[type="datetime"], .wy-control-group.fluid-input input[type="datetime-local"], .wy-control-group.fluid-input input[type="week"], .wy-control-group.fluid-input input[type="number"], .wy-control-group.fluid-input input[type="search"], .wy-control-group.fluid-input input[type="tel"], .wy-control-group.fluid-input input[type="color"] {
  width: 100%;
}

.wy-form-message-inline {
  display: inline-block;
  padding-left: 0.3em;
  color: #666666;
  vertical-align: middle;
  font-size: 90%;
}

.wy-form-message {
  display: block;
  color: #cccccc;
  font-size: 70%;
  margin-top: 0.3125em;
  font-style: italic;
}

.wy-tag-input-group {
  padding: 4px 4px 0px 4px;
  display: inline-block;
  border: 1px solid #cccccc;
  font-size: 80%;
  font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
  box-shadow: inset 0 1px 3px #dddddd;
  background: white;
  -webkit-transition: border 0.3s linear;
  -moz-transition: border 0.3s linear;
  transition: border 0.3s linear;
}
.wy-tag-input-group .wy-tag {
  display: inline-block;
  background-color: rgba(0, 0, 0, 0.1);
  padding: 0.5em 0.625em;
  border-radius: 2px;
  position: relative;
  margin-bottom: 4px;
}
.wy-tag-input-group .wy-tag .wy-tag-remove {
  color: #cccccc;
  margin-left: 5px;
}
.wy-tag-input-group .wy-tag .wy-tag-remove:hover {
  color: #e74c3c;
}
.wy-tag-input-group label {
  margin-left: 5px;
  display: inline-block;
  margin-bottom: 0;
}
.wy-tag-input-group input {
  border: none;
  font-size: 100%;
  margin-bottom: 4px;
  box-shadow: none;
}

.wy-form-upload {
  border: solid 1px #cccccc;
  border-bottom: solid 3px #cccccc;
  background-color: #f3f6f6;
  padding: 24px;
  display: inline-block;
  text-align: center;
  cursor: pointer;
  color: #404040;
  -webkit-transition: border-color 0.1s ease-in;
  -moz-transition: border-color 0.1s ease-in;
  transition: border-color 0.1s ease-in;
  *zoom: 1;
}
.wy-form-upload:before, .wy-form-upload:after {
  display: table;
  content: "";
}
.wy-form-upload:after {
  clear: both;
}
@media screen and (max-width: 480px) {
  .wy-form-upload {
    width: 100%;
  }
}
.wy-form-upload .image-drop {
  display: none;
}
.wy-form-upload .image-desktop {
  display: none;
}
.wy-form-upload .image-loading {
  display: none;
}
.wy-form-upload .wy-form-upload-icon {
  display: block;
  font-size: 32px;
  color: #b3b3b3;
}
.wy-form-upload .image-drop .wy-form-upload-icon {
  color: #27ae60;
}
.wy-form-upload p {
  font-size: 90%;
}
.wy-form-upload .wy-form-upload-image {
  float: left;
  margin-right: 24px;
}
@media screen and (max-width: 480px) {
  .wy-form-upload .wy-form-upload-image {
    width: 100%;
    margin-bottom: 24px;
  }
}
.wy-form-upload img {
  max-width: 125px;
  max-height: 125px;
  opacity: 0.9;
  -webkit-transition: opacity 0.1s ease-in;
  -moz-transition: opacity 0.1s ease-in;
  transition: opacity 0.1s ease-in;
}
.wy-form-upload .wy-form-upload-content {
  float: left;
}
@media screen and (max-width: 480px) {
  .wy-form-upload .wy-form-upload-content {
    width: 100%;
  }
}

.wy-form-upload:hover {
  border-color: #b3b3b3;
  color: #404040;
}
.wy-form-upload:hover .image-desktop {
  display: block;
}
.wy-form-upload:hover .image-drag {
  display: none;
}
.wy-form-upload:hover img {
  opacity: 1;
}

.wy-form-upload:active {
  border-top: solid 3px #cccccc;
  border-bottom: solid 1px #cccccc;
}

.wy-form-upload.wy-form-upload-big {
  width: 100%;
  text-align: center;
  padding: 72px;
}
.wy-form-upload.wy-form-upload-big .wy-form-upload-content {
  float: none;
}

.wy-form-upload.wy-form-upload-file p {
  margin-bottom: 0;
}
.wy-form-upload.wy-form-upload-file .wy-form-upload-icon {
  display: inline-block;
  font-size: inherit;
}

.wy-form-upload.wy-form-upload-drop {
  background-color: #ddf7e8;
}
.wy-form-upload.wy-form-upload-drop .image-drop {
  display: block;
}
.wy-form-upload.wy-form-upload-drop .image-desktop {
  display: none;
}
.wy-form-upload.wy-form-upload-drop .image-drag {
  display: none;
}

.wy-form-upload.wy-form-upload-loading .image-drag {
  display: none;
}
.wy-form-upload.wy-form-upload-loading .image-desktop {
  display: none;
}
.wy-form-upload.wy-form-upload-loading .image-loading {
  display: block;
}
.wy-form-upload.wy-form-upload-loading .wy-input-prefix {
  display: none;
}
.wy-form-upload.wy-form-upload-loading p {
  margin-bottom: 0;
}

.rotate-90 {
  -webkit-transform: rotate(90deg);
  -moz-transform: rotate(90deg);
  -ms-transform: rotate(90deg);
  -o-transform: rotate(90deg);
  transform: rotate(90deg);
}

.rotate-180 {
  -webkit-transform: rotate(180deg);
  -moz-transform: rotate(180deg);
  -ms-transform: rotate(180deg);
  -o-transform: rotate(180deg);
  transform: rotate(180deg);
}

.rotate-270 {
  -webkit-transform: rotate(270deg);
  -moz-transform: rotate(270deg);
  -ms-transform: rotate(270deg);
  -o-transform: rotate(270deg);
  transform: rotate(270deg);
}

.mirror {
  -webkit-transform: scaleX(-1);
  -moz-transform: scaleX(-1);
  -ms-transform: scaleX(-1);
  -o-transform: scaleX(-1);
  transform: scaleX(-1);
}
.mirror.rotate-90 {
  -webkit-transform: scaleX(-1) rotate(90deg);
  -moz-transform: scaleX(-1) rotate(90deg);
  -ms-transform: scaleX(-1) rotate(90deg);
  -o-transform: scaleX(-1) rotate(90deg);
  transform: scaleX(-1) rotate(90deg);
}
.mirror.rotate-180 {
  -webkit-transform: scaleX(-1) rotate(180deg);
  -moz-transform: scaleX(-1) rotate(180deg);
  -ms-transform: scaleX(-1) rotate(180deg);
  -o-transform: scaleX(-1) rotate(180deg);
  transform: scaleX(-1) rotate(180deg);
}
.mirror.rotate-270 {
  -webkit-transform: scaleX(-1) rotate(270deg);
  -moz-transform: scaleX(-1) rotate(270deg);
  -ms-transform: scaleX(-1) rotate(270deg);
  -o-transform: scaleX(-1) rotate(270deg);
  transform: scaleX(-1) rotate(270deg);
}

.wy-form-gallery-manage {
  margin-left: -12px;
  margin-right: -12px;
}
.wy-form-gallery-manage li {
  float: left;
  padding: 12px;
  width: 20%;
  cursor: pointer;
}
@media screen and (max-width: 768px) {
  .wy-form-gallery-manage li {
    width: 25%;
  }
}
@media screen and (max-width: 480px) {
  .wy-form-gallery-manage li {
    width: 50%;
  }
}
.wy-form-gallery-manage li:active {
  cursor: move;
}
.wy-form-gallery-manage li > a {
  padding: 12px;
  background-color: white;
  border: solid 1px #e1e4e5;
  border-bottom: solid 3px #e1e4e5;
  display: inline-block;
  -webkit-transition: all 0.1s ease-in;
  -moz-transition: all 0.1s ease-in;
  transition: all 0.1s ease-in;
}
.wy-form-gallery-manage li > a:active {
  border: solid 1px #cccccc;
  border-top: solid 3px #cccccc;
}
.wy-form-gallery-manage img {
  width: 100%;
  -webkit-transition: all 0.05s ease-in;
  -moz-transition: all 0.05s ease-in;
  transition: all 0.05s ease-in;
}

li.wy-form-gallery-edit {
  position: relative;
  color: white;
  padding: 24px;
  width: 100%;
  display: block;
  background-color: #343131;
  border-radius: 4px;
}
li.wy-form-gallery-edit .arrow {
  position: absolute;
  display: block;
  top: -50px;
  left: 50%;
  margin-left: -25px;
  z-index: 500;
  height: 0;
  width: 0;
  border-color: transparent;
  border-style: solid;
  border-width: 25px;
  border-bottom-color: #343131;
}

@media only screen and (max-width: 480px) {
  .wy-form button[type="submit"] {
    margin: 0.7em 0 0;
  }
  .wy-form input[type="text"], .wy-form input[type="password"], .wy-form input[type="email"], .wy-form input[type="url"], .wy-form input[type="date"], .wy-form input[type="month"], .wy-form input[type="time"], .wy-form input[type="datetime"], .wy-form input[type="datetime-local"], .wy-form input[type="week"], .wy-form input[type="number"], .wy-form input[type="search"], .wy-form input[type="tel"], .wy-form input[type="color"] {
    margin-bottom: 0.3em;
    display: block;
  }
  .wy-form label {
    margin-bottom: 0.3em;
    display: block;
  }

  .wy-form input[type="password"], .wy-form input[type="email"], .wy-form input[type="url"], .wy-form input[type="date"], .wy-form input[type="month"], .wy-form input[type="time"], .wy-form input[type="datetime"], .wy-form input[type="datetime-local"], .wy-form input[type="week"], .wy-form input[type="number"], .wy-form input[type="search"], .wy-form input[type="tel"], .wy-form input[type="color"] {
    margin-bottom: 0;
  }

  .wy-form-aligned .wy-control-group label {
    margin-bottom: 0.3em;
    text-align: left;
    display: block;
    width: 100%;
  }
  .wy-form-aligned .wy-controls {
    margin: 1.5em 0 0 0;
  }

  /* NOTE: wy-help-inline is deprecated. Use .wy-form-message-inline instead. */
  .wy-form .wy-help-inline, .wy-form-message-inline, .wy-form-message {
    display: block;
    font-size: 80%;
    /* Increased bottom padding to make it group with its related input element. */
    padding: 0.2em 0 0.8em;
  }
}
@media screen and (max-width: 768px) {
  .tablet-hide {
    display: none;
  }
}

@media screen and (max-width: 480px) {
  .mobile-hide {
    display: none;
  }
}

.float-left {
  float: left;
}

.float-right {
  float: right;
}

.full-width {
  width: 100%;
}

.wy-grid-one-col {
  *zoom: 1;
  max-width: 68em;
  margin-left: auto;
  margin-right: auto;
  max-width: 1066px;
  margin-top: 1.618em;
}
.wy-grid-one-col:before, .wy-grid-one-col:after {
  display: table;
  content: "";
}
.wy-grid-one-col:after {
  clear: both;
}
.wy-grid-one-col section {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 100%;
  background: #fcfcfc;
  padding: 1.618em;
  margin-right: 0;
}
.wy-grid-one-col section:last-child {
  margin-right: 0;
}

.wy-grid-index-card {
  *zoom: 1;
  max-width: 68em;
  margin-left: auto;
  margin-right: auto;
  max-width: 460px;
  margin-top: 1.618em;
  background: #fcfcfc;
  padding: 1.618em;
}
.wy-grid-index-card:before, .wy-grid-index-card:after {
  display: table;
  content: "";
}
.wy-grid-index-card:after {
  clear: both;
}
.wy-grid-index-card header, .wy-grid-index-card section, .wy-grid-index-card aside {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 100%;
}
.wy-grid-index-card header:last-child, .wy-grid-index-card section:last-child, .wy-grid-index-card aside:last-child {
  margin-right: 0;
}
.wy-grid-index-card.twocol {
  max-width: 768px;
}
.wy-grid-index-card.twocol section {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 48.82117%;
}
.wy-grid-index-card.twocol section:last-child {
  margin-right: 0;
}
.wy-grid-index-card.twocol aside {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 48.82117%;
}
.wy-grid-index-card.twocol aside:last-child {
  margin-right: 0;
}

.wy-grid-search-filter {
  *zoom: 1;
  max-width: 68em;
  margin-left: auto;
  margin-right: auto;
  margin-bottom: 24px;
}
.wy-grid-search-filter:before, .wy-grid-search-filter:after {
  display: table;
  content: "";
}
.wy-grid-search-filter:after {
  clear: both;
}
.wy-grid-search-filter .wy-grid-search-filter-input {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 74.41059%;
}
.wy-grid-search-filter .wy-grid-search-filter-input:last-child {
  margin-right: 0;
}
.wy-grid-search-filter .wy-grid-search-filter-btn {
  display: block;
  float: left;
  margin-right: 2.35765%;
  width: 23.23176%;
}
.wy-grid-search-filter .wy-grid-search-filter-btn:last-child {
  margin-right: 0;
}

.wy-table, .rst-content table.docutils, .rst-content table.field-list {
  /* Remove spacing between table cells (from Normalize.css) */
  border-collapse: collapse;
  border-spacing: 0;
  empty-cells: show;
  margin-bottom: 24px;
}
.wy-table caption, .rst-content table.docutils caption, .rst-content table.field-list caption {
  color: black;
  font: italic 85%/1 arial, sans-serif;
  padding: 1em 0;
  text-align: center;
}
.wy-table td, .rst-content table.docutils td, .rst-content table.field-list td, .wy-table th, .rst-content table.docutils th, .rst-content table.field-list th {
  font-size: 90%;
  margin: 0;
  overflow: visible;
  padding: 8px 16px;
}
.wy-table td:first-child, .rst-content table.docutils td:first-child, .rst-content table.field-list td:first-child, .wy-table th:first-child, .rst-content table.docutils th:first-child, .rst-content table.field-list th:first-child {
  border-left-width: 0;
}
.wy-table thead, .rst-content table.docutils thead, .rst-content table.field-list thead {
  color: black;
  text-align: left;
  vertical-align: bottom;
  white-space: nowrap;
}
.wy-table thead th, .rst-content table.docutils thead th, .rst-content table.field-list thead th {
  font-weight: bold;
  border-bottom: solid 2px #e1e4e5;
}
.wy-table td, .rst-content table.docutils td, .rst-content table.field-list td {
  background-color: transparent;
  vertical-align: middle;
}

.wy-table td p, .rst-content table.docutils td p, .rst-content table.field-list td p {
  line-height: 18px;
  margin-bottom: 0;
}

.wy-table .wy-table-cell-min, .rst-content table.docutils .wy-table-cell-min, .rst-content table.field-list .wy-table-cell-min {
  width: 1%;
  padding-right: 0;
}
.wy-table .wy-table-cell-min input[type=checkbox], .rst-content table.docutils .wy-table-cell-min input[type=checkbox], .rst-content table.field-list .wy-table-cell-min input[type=checkbox], .wy-table .wy-table-cell-min input[type=checkbox], .rst-content table.docutils .wy-table-cell-min input[type=checkbox], .rst-content table.field-list .wy-table-cell-min input[type=checkbox] {
  margin: 0;
}

.wy-table-secondary {
  color: gray;
  font-size: 90%;
}

.wy-table-tertiary {
  color: gray;
  font-size: 80%;
}

/*striping:
 *   even - #fff (white)
 *   odd  - #f2f2f2 (light gray) */
.wy-table-odd td, .wy-table-striped tr:nth-child(2n-1) td, .rst-content table.docutils:not(.field-list) tr:nth-child(2n-1) td {
  background-color: #f3f6f6;
}

.wy-table-backed {
  background-color: #f3f6f6;
}

/* nth-child selector for modern browsers */
/* BORDERED TABLES */
.wy-table-bordered-all, .rst-content table.docutils {
  border: 1px solid #e1e4e5;
}
.wy-table-bordered-all td, .rst-content table.docutils td {
  border-bottom: 1px solid #e1e4e5;
  border-left: 1px solid #e1e4e5;
}
.wy-table-bordered-all tbody > tr:last-child td, .rst-content table.docutils tbody > tr:last-child td {
  border-bottom-width: 0;
}

.wy-table-bordered {
  border: 1px solid #e1e4e5;
}

.wy-table-bordered-rows td {
  border-bottom: 1px solid #e1e4e5;
}
.wy-table-bordered-rows tbody > tr:last-child td {
  border-bottom-width: 0;
}

.wy-table-horizontal tbody > tr:last-child td {
  border-bottom-width: 0;
}
.wy-table-horizontal td, .wy-table-horizontal th {
  border-width: 0 0 1px 0;
  border-bottom: 1px solid #e1e4e5;
}
.wy-table-horizontal tbody > tr:last-child td {
  border-bottom-width: 0;
}

/* RESPONSIVE TABLES */
.wy-table-responsive {
  margin-bottom: 24px;
  max-width: 100%;
  overflow: auto;
}
.wy-table-responsive table {
  margin-bottom: 0 !important;
}
.wy-table-responsive table td, .wy-table-responsive table th {
  white-space: nowrap;
}

html {
  height: 100%;
  overflow-x: hidden;
}

body {
  font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
  font-weight: normal;
  color: #404040;
  min-height: 100%;
  overflow-x: hidden;
  background: #edf0f2;
}

a {
  color: #2980b9;
  text-decoration: none;
}
a:hover {
  color: #3091d1;
}
a:visited {
  color: #9b59b6;
}

.link-danger {
  color: #e74c3c;
}
.link-danger:hover {
  color: #d62c1a;
}

.text-left {
  text-align: left;
}

.text-center {
  text-align: center;
}

.text-right {
  text-align: right;
}

h1, h2, h3, h4, h5, h6, legend {
  margin-top: 0;
  font-weight: 700;
  font-family: "Roboto Slab", "ff-tisa-web-pro", "Georgia", Arial, sans-serif;
}

p {
  line-height: 24px;
  margin: 0;
  font-size: 16px;
  margin-bottom: 24px;
}

h1 {
  font-size: 175%;
}

h2 {
  font-size: 150%;
}

h3 {
  font-size: 125%;
}

h4 {
  font-size: 115%;
}

h5 {
  font-size: 110%;
}

h6 {
  font-size: 100%;
}

small {
  font-size: 80%;
}

code, .rst-content tt {
  white-space: nowrap;
  max-width: 100%;
  background: white;
  border: solid 1px #e1e4e5;
  font-size: 75%;
  padding: 0 5px;
  font-family: "Incosolata", "Consolata", "Monaco", monospace;
  color: #e74c3c;
  overflow-x: auto;
}
code.code-large, .rst-content tt.code-large {
  font-size: 90%;
}

.full-width {
  width: 100%;
}

.wy-plain-list-disc, .rst-content .section ul, .rst-content .toctree-wrapper ul {
  list-style: disc;
  line-height: 24px;
  margin-bottom: 24px;
}

.wy-plain-list-disc li, .rst-content .section ul li, .rst-content .toctree-wrapper ul li {
  list-style: disc;
  margin-left: 24px;
}
.wy-plain-list-disc li ul, .rst-content .section ul li ul, .rst-content .toctree-wrapper ul li ul {
  margin-bottom: 0;
}
.wy-plain-list-disc li li, .rst-content .section ul li li, .rst-content .toctree-wrapper ul li li {
  list-style: circle;
}
.wy-plain-list-disc li li li, .rst-content .section ul li li li, .rst-content .toctree-wrapper ul li li li {
  list-style: square;
}

.wy-plain-list-decimal, .rst-content .section ol, .rst-content ol.arabic {
  list-style: decimal;
  line-height: 24px;
  margin-bottom: 24px;
}

.wy-plain-list-decimal li, .rst-content .section ol li, .rst-content ol.arabic li {
  list-style: decimal;
  margin-left: 24px;
}

.wy-type-large {
  font-size: 120%;
}

.wy-type-normal {
  font-size: 100%;
}

.wy-type-small {
  font-size: 100%;
}

.wy-type-strike {
  text-decoration: line-through;
}

.wy-text-warning {
  color: #e67e22 !important;
}

a.wy-text-warning:hover {
  color: #eb9950 !important;
}

.wy-text-info {
  color: #2980b9 !important;
}

a.wy-text-info:hover {
  color: #409ad5 !important;
}

.wy-text-success {
  color: #27ae60 !important;
}

a.wy-text-success:hover {
  color: #36d278 !important;
}

.wy-text-danger {
  color: #e74c3c !important;
}

a.wy-text-danger:hover {
  color: #ed7669 !important;
}

.wy-text-neutral {
  color: #404040 !important;
}

a.wy-text-neutral:hover {
  color: #595959 !important;
}

.codeblock-example {
  border: 1px solid #e1e4e5;
  border-bottom: none;
  padding: 24px;
  padding-top: 48px;
  font-weight: 500;
  background: white;
  position: relative;
}
.codeblock-example:after {
  content: "Example";
  position: absolute;
  top: 0px;
  left: 0px;
  background: #9b59b6;
  color: white;
  padding: 6px 12px;
}
.codeblock-example.prettyprint-example-only {
  border: 1px solid #e1e4e5;
  margin-bottom: 24px;
}

.codeblock, pre.literal-block, .rst-content .literal-block, div[class^='highlight'] {
  border: 1px solid #e1e4e5;
  padding: 0px;
  overflow-x: auto;
  background: white;
  margin: 1px 0 24px 0;
}
.codeblock div[class^='highlight'], pre.literal-block div[class^='highlight'], .rst-content .literal-block div[class^='highlight'], div[class^='highlight'] div[class^='highlight'] {
  border: none;
  background: none;
  margin: 0;
}

div[class^='highlight'] td.code {
  width: 100%;
}

.linenodiv pre {
  border-right: solid 1px #e6e9ea;
  margin: 0;
  padding: 12px 12px;
  font-family: "Incosolata", "Consolata", "Monaco", monospace;
  font-size: 12px;
  line-height: 1.5;
  color: #d9d9d9;
}

div[class^='highlight'] pre {
  white-space: pre;
  margin: 0;
  padding: 12px 12px;
  font-family: "Incosolata", "Consolata", "Monaco", monospace;
  font-size: 12px;
  line-height: 1.5;
  display: block;
  overflow: auto;
  color: #404040;
}

@media print {
  .codeblock, pre.literal-block, .rst-content .literal-block, div[class^='highlight'], div[class^='highlight'] pre {
    white-space: pre-wrap;
  }
}
.hll {
  background-color: #ffffcc;
  margin: 0 -12px;
  padding: 0 12px;
  display: block;
}

.c {
  color: #999988;
  font-style: italic;
}

.err {
  color: #a61717;
  background-color: #e3d2d2;
}

.k {
  font-weight: bold;
}

.o {
  font-weight: bold;
}

.cm {
  color: #999988;
  font-style: italic;
}

.cp {
  color: #999999;
  font-weight: bold;
}

.c1 {
  color: #999988;
  font-style: italic;
}

.cs {
  color: #999999;
  font-weight: bold;
  font-style: italic;
}

.gd {
  color: black;
  background-color: #ffdddd;
}

.gd .x {
  color: black;
  background-color: #ffaaaa;
}

.ge {
  font-style: italic;
}

.gr {
  color: #aa0000;
}

.gh {
  color: #999999;
}

.gi {
  color: black;
  background-color: #ddffdd;
}

.gi .x {
  color: black;
  background-color: #aaffaa;
}

.go {
  color: #888888;
}

.gp {
  color: #555555;
}

.gs {
  font-weight: bold;
}

.gu {
  color: purple;
  font-weight: bold;
}

.gt {
  color: #aa0000;
}

.kc {
  font-weight: bold;
}

.kd {
  font-weight: bold;
}

.kn {
  font-weight: bold;
}

.kp {
  font-weight: bold;
}

.kr {
  font-weight: bold;
}

.kt {
  color: #445588;
  font-weight: bold;
}

.m {
  color: #009999;
}

.s {
  color: #dd1144;
}

.n {
  color: #333333;
}

.na {
  color: teal;
}

.nb {
  color: #0086b3;
}

.nc {
  color: #445588;
  font-weight: bold;
}

.no {
  color: teal;
}

.ni {
  color: purple;
}

.ne {
  color: #990000;
  font-weight: bold;
}

.nf {
  color: #990000;
  font-weight: bold;
}

.nn {
  color: #555555;
}

.nt {
  color: navy;
}

.nv {
  color: teal;
}

.ow {
  font-weight: bold;
}

.w {
  color: #bbbbbb;
}

.mf {
  color: #009999;
}

.mh {
  color: #009999;
}

.mi {
  color: #009999;
}

.mo {
  color: #009999;
}

.sb {
  color: #dd1144;
}

.sc {
  color: #dd1144;
}

.sd {
  color: #dd1144;
}

.s2 {
  color: #dd1144;
}

.se {
  color: #dd1144;
}

.sh {
  color: #dd1144;
}

.si {
  color: #dd1144;
}

.sx {
  color: #dd1144;
}

.sr {
  color: #009926;
}

.s1 {
  color: #dd1144;
}

.ss {
  color: #990073;
}

.bp {
  color: #999999;
}

.vc {
  color: teal;
}

.vg {
  color: teal;
}

.vi {
  color: teal;
}

.il {
  color: #009999;
}

.gc {
  color: #999999;
  background-color: #eaf2f5;
}

.wy-breadcrumbs li {
  display: inline-block;
}
.wy-breadcrumbs li.wy-breadcrumbs-aside {
  float: right;
}
.wy-breadcrumbs li a {
  display: inline-block;
  padding: 5px;
}
.wy-breadcrumbs li a:first-child {
  padding-left: 0;
}

.wy-breadcrumbs-extra {
  margin-bottom: 0;
  color: #b3b3b3;
  font-size: 80%;
  display: inline-block;
}

@media screen and (max-width: 480px) {
  .wy-breadcrumbs-extra {
    display: none;
  }

  .wy-breadcrumbs li.wy-breadcrumbs-aside {
    display: none;
  }
}
@media print {
  .wy-breadcrumbs li.wy-breadcrumbs-aside {
    display: none;
  }
}
.wy-affix {
  position: fixed;
  top: 1.618em;
}

.wy-menu a:hover {
  text-decoration: none;
}

.wy-menu-horiz {
  *zoom: 1;
}
.wy-menu-horiz:before, .wy-menu-horiz:after {
  display: table;
  content: "";
}
.wy-menu-horiz:after {
  clear: both;
}
.wy-menu-horiz ul, .wy-menu-horiz li {
  display: inline-block;
}
.wy-menu-horiz li:hover {
  background: rgba(255, 255, 255, 0.1);
}
.wy-menu-horiz li.divide-left {
  border-left: solid 1px #404040;
}
.wy-menu-horiz li.divide-right {
  border-right: solid 1px #404040;
}
.wy-menu-horiz a {
  height: 32px;
  display: inline-block;
  line-height: 32px;
  padding: 0 16px;
}

.wy-menu-vertical header {
  height: 32px;
  display: inline-block;
  line-height: 32px;
  padding: 0 1.618em;
  display: block;
  font-weight: bold;
  text-transform: uppercase;
  font-size: 80%;
  color: #2980b9;
  white-space: nowrap;
}
.wy-menu-vertical ul {
  margin-bottom: 0;
}
.wy-menu-vertical li.divide-top {
  border-top: solid 1px #404040;
}
.wy-menu-vertical li.divide-bottom {
  border-bottom: solid 1px #404040;
}
.wy-menu-vertical li.current {
  background: #e3e3e3;
}
.wy-menu-vertical li.current a {
  color: gray;
  border-right: solid 1px #c9c9c9;
  padding: 0.4045em 2.427em;
}
.wy-menu-vertical li.current a:hover {
  background: #d6d6d6;
}
.wy-menu-vertical li.on a, .wy-menu-vertical li.current > a {
  color: #404040;
  padding: 0.4045em 1.618em;
  font-weight: bold;
  position: relative;
  background: #fcfcfc;
  border: none;
  border-bottom: solid 1px #c9c9c9;
  border-top: solid 1px #c9c9c9;
  padding-left: 1.618em -4px;
}
.wy-menu-vertical li.on a:hover, .wy-menu-vertical li.current > a:hover {
  background: #fcfcfc;
}
.wy-menu-vertical li.toctree-l2.current > a {
  background: #c9c9c9;
  padding: 0.4045em 2.427em;
}
.wy-menu-vertical li.current ul {
  display: block;
}
.wy-menu-vertical li ul {
  margin-bottom: 0;
  display: none;
}
.wy-menu-vertical .local-toc li ul {
  display: block;
}
.wy-menu-vertical li ul li a {
  margin-bottom: 0;
  color: #b3b3b3;
  font-weight: normal;
}
.wy-menu-vertical a {
  display: inline-block;
  line-height: 18px;
  padding: 0.4045em 1.618em;
  display: block;
  position: relative;
  font-size: 90%;
  color: #b3b3b3;
}
.wy-menu-vertical a:hover {
  background-color: #4e4a4a;
  cursor: pointer;
}
.wy-menu-vertical a:active {
  background-color: #2980b9;
  cursor: pointer;
  color: white;
}

.wy-side-nav-search {
  z-index: 200;
  background-color: #2980b9;
  text-align: center;
  padding: 0.809em;
  display: block;
  color: #fcfcfc;
  margin-bottom: 0.809em;
}
.wy-side-nav-search input[type=text] {
  width: 100%;
  border-radius: 50px;
  padding: 6px 12px;
  border-color: #2472a4;
}
.wy-side-nav-search img {
  display: block;
  margin: auto auto 0.809em auto;
  height: 45px;
  width: 45px;
  background-color: #2980b9;
  padding: 5px;
  border-radius: 100%;
}
.wy-side-nav-search > a, .wy-side-nav-search .wy-dropdown > a {
  color: #fcfcfc;
  font-size: 100%;
  font-weight: bold;
  display: inline-block;
  padding: 4px 6px;
  margin-bottom: 0.809em;
}
.wy-side-nav-search > a:hover, .wy-side-nav-search .wy-dropdown > a:hover {
  background: rgba(255, 255, 255, 0.1);
}

.wy-nav .wy-menu-vertical header {
  color: #2980b9;
}
.wy-nav .wy-menu-vertical a {
  color: #b3b3b3;
}
.wy-nav .wy-menu-vertical a:hover {
  background-color: #2980b9;
  color: white;
}

[data-menu-wrap] {
  -webkit-transition: all 0.2s ease-in;
  -moz-transition: all 0.2s ease-in;
  transition: all 0.2s ease-in;
  position: absolute;
  opacity: 1;
  width: 100%;
  opacity: 0;
}
[data-menu-wrap].move-center {
  left: 0;
  right: auto;
  opacity: 1;
}
[data-menu-wrap].move-left {
  right: auto;
  left: -100%;
  opacity: 0;
}
[data-menu-wrap].move-right {
  right: -100%;
  left: auto;
  opacity: 0;
}

.wy-body-for-nav {
  background: left repeat-y #fcfcfc;
  background-image: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAIAAACQd1PeAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyRpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuMy1jMDExIDY2LjE0NTY2MSwgMjAxMi8wMi8wNi0xNDo1NjoyNyAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENTNiAoTWFjaW50b3NoKSIgeG1wTU06SW5zdGFuY2VJRD0ieG1wLmlpZDoxOERBMTRGRDBFMUUxMUUzODUwMkJCOThDMEVFNURFMCIgeG1wTU06RG9jdW1lbnRJRD0ieG1wLmRpZDoxOERBMTRGRTBFMUUxMUUzODUwMkJCOThDMEVFNURFMCI+IDx4bXBNTTpEZXJpdmVkRnJvbSBzdFJlZjppbnN0YW5jZUlEPSJ4bXAuaWlkOjE4REExNEZCMEUxRTExRTM4NTAyQkI5OEMwRUU1REUwIiBzdFJlZjpkb2N1bWVudElEPSJ4bXAuZGlkOjE4REExNEZDMEUxRTExRTM4NTAyQkI5OEMwRUU1REUwIi8+IDwvcmRmOkRlc2NyaXB0aW9uPiA8L3JkZjpSREY+IDwveDp4bXBtZXRhPiA8P3hwYWNrZXQgZW5kPSJyIj8+EwrlwAAAAA5JREFUeNpiMDU0BAgwAAE2AJgB9BnaAAAAAElFTkSuQmCC);
  background-size: 300px 1px;
}

.wy-grid-for-nav {
  position: absolute;
  width: 100%;
  height: 100%;
}

.wy-nav-side {
  position: absolute;
  top: 0;
  left: 0;
  width: 300px;
  overflow: hidden;
  min-height: 100%;
  background: #343131;
  z-index: 200;
}

.wy-nav-top {
  display: none;
  background: #2980b9;
  color: white;
  padding: 0.4045em 0.809em;
  position: relative;
  line-height: 50px;
  text-align: center;
  font-size: 100%;
  *zoom: 1;
}
.wy-nav-top:before, .wy-nav-top:after {
  display: table;
  content: "";
}
.wy-nav-top:after {
  clear: both;
}
.wy-nav-top a {
  color: white;
  font-weight: bold;
}
.wy-nav-top img {
  margin-right: 12px;
  height: 45px;
  width: 45px;
  background-color: #2980b9;
  padding: 5px;
  border-radius: 100%;
}
.wy-nav-top i {
  font-size: 30px;
  float: left;
  cursor: pointer;
}

.wy-nav-content-wrap {
  margin-left: 300px;
  background: #fcfcfc;
  min-height: 100%;
}

.wy-nav-content {
  padding: 1.618em 3.236em;
  height: 100%;
  max-width: 800px;
  margin: auto;
}

.wy-body-mask {
  position: fixed;
  width: 100%;
  height: 100%;
  background: rgba(0, 0, 0, 0.2);
  display: none;
  z-index: 499;
}
.wy-body-mask.on {
  display: block;
}

footer {
  color: #999999;
}
footer p {
  margin-bottom: 12px;
}

.rst-footer-buttons {
  *zoom: 1;
}
.rst-footer-buttons:before, .rst-footer-buttons:after {
  display: table;
  content: "";
}
.rst-footer-buttons:after {
  clear: both;
}

#search-results .search li {
  margin-bottom: 24px;
  border-bottom: solid 1px #e1e4e5;
  padding-bottom: 24px;
}
#search-results .search li:first-child {
  border-top: solid 1px #e1e4e5;
  padding-top: 24px;
}
#search-results .search li a {
  font-size: 120%;
  margin-bottom: 12px;
  display: inline-block;
}
#search-results .context {
  color: gray;
  font-size: 90%;
}

@media screen and (max-width: 768px) {
  .wy-body-for-nav {
    background: #fcfcfc;
  }

  .wy-nav-top {
    display: block;
  }

  .wy-nav-side {
    left: -300px;
  }
  .wy-nav-side.shift {
    width: 85%;
    left: 0;
  }

  .wy-nav-content-wrap {
    margin-left: 0;
  }
  .wy-nav-content-wrap .wy-nav-content {
    padding: 1.618em;
  }
  .wy-nav-content-wrap.shift {
    position: fixed;
    min-width: 100%;
    left: 85%;
    top: 0;
    height: 100%;
    overflow: hidden;
  }
}
@media screen and (min-width: 1400px) {
  .wy-nav-content-wrap {
    background: rgba(0, 0, 0, 0.05);
  }

  .wy-nav-content {
    margin: 0;
    background: #fcfcfc;
  }
}
@media print {
  .wy-nav-side {
    display: none;
  }

  .wy-nav-content-wrap {
    margin-left: 0;
  }
}
nav.stickynav {
  position: fixed;
  top: 0;
}

.rst-versions {
  position: fixed;
  bottom: 0;
  left: 0;
  width: 300px;
  color: #fcfcfc;
  background: #1f1d1d;
  border-top: solid 10px #343131;
  font-family: "Lato", "proxima-nova", "Helvetica Neue", Arial, sans-serif;
  z-index: 400;
}
.rst-versions a {
  color: #2980b9;
  text-decoration: none;
}
.rst-versions .rst-badge-small {
  display: none;
}
.rst-versions .rst-current-version {
  padding: 12px;
  background-color: #272525;
  display: block;
  text-align: right;
  font-size: 90%;
  cursor: pointer;
  color: #27ae60;
  *zoom: 1;
}
.rst-versions .rst-current-version:before, .rst-versions .rst-current-version:after {
  display: table;
  content: "";
}
.rst-versions .rst-current-version:after {
  clear: both;
}
.rst-versions .rst-current-version .fa, .rst-versions .rst-current-version .wy-dropdown .caret, .wy-dropdown .rst-versions .rst-current-version .caret, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-success .wy-input-context, .wy-inline-validate.wy-inline-validate-success .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-danger .wy-input-context, .wy-inline-validate.wy-inline-validate-danger .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-warning .wy-input-context, .wy-inline-validate.wy-inline-validate-warning .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-inline-validate.wy-inline-validate-info .wy-input-context, .wy-inline-validate.wy-inline-validate-info .rst-versions .rst-current-version .wy-input-context, .rst-versions .rst-current-version .wy-tag-input-group .wy-tag .wy-tag-remove, .wy-tag-input-group .wy-tag .rst-versions .rst-current-version .wy-tag-remove, .rst-versions .rst-current-version .rst-content .admonition-title, .rst-content .rst-versions .rst-current-version .admonition-title, .rst-versions .rst-current-version .rst-content h1 .headerlink, .rst-content h1 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h2 .headerlink, .rst-content h2 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h3 .headerlink, .rst-content h3 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h4 .headerlink, .rst-content h4 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h5 .headerlink, .rst-content h5 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content h6 .headerlink, .rst-content h6 .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .rst-content dl dt .headerlink, .rst-content dl dt .rst-versions .rst-current-version .headerlink, .rst-versions .rst-current-version .icon {
  color: #fcfcfc;
}
.rst-versions .rst-current-version .fa-book, .rst-versions .rst-current-version .icon-book {
  float: left;
}
.rst-versions .rst-current-version .icon-book {
  float: left;
}
.rst-versions .rst-current-version.rst-out-of-date {
  background-color: #e74c3c;
  color: white;
}
.rst-versions .rst-current-version.rst-active-old-version {
  background-color: #f1c40f;
  color: black;
}
.rst-versions.shift-up .rst-other-versions {
  display: block;
}
.rst-versions .rst-other-versions {
  font-size: 90%;
  padding: 12px;
  color: gray;
  display: none;
}
.rst-versions .rst-other-versions hr {
  display: block;
  height: 1px;
  border: 0;
  margin: 20px 0;
  padding: 0;
  border-top: solid 1px #413d3d;
}
.rst-versions .rst-other-versions dd {
  display: inline-block;
  margin: 0;
}
.rst-versions .rst-other-versions dd a {
  display: inline-block;
  padding: 6px;
  color: #fcfcfc;
}
.rst-versions.rst-badge {
  width: auto;
  bottom: 20px;
  right: 20px;
  left: auto;
  border: none;
  max-width: 300px;
}
.rst-versions.rst-badge .icon-book {
  float: none;
}
.rst-versions.rst-badge .fa-book, .rst-versions.rst-badge .icon-book {
  float: none;
}
.rst-versions.rst-badge.shift-up .rst-current-version {
  text-align: right;
}
.rst-versions.rst-badge.shift-up .rst-current-version .fa-book, .rst-versions.rst-badge.shift-up .rst-current-version .icon-book {
  float: left;
}
.rst-versions.rst-badge.shift-up .rst-current-version .icon-book {
  float: left;
}
.rst-versions.rst-badge .rst-current-version {
  width: auto;
  height: 30px;
  line-height: 30px;
  padding: 0 6px;
  display: block;
  text-align: center;
}

@media screen and (max-width: 768px) {
  .rst-versions {
    width: 85%;
    display: none;
  }
  .rst-versions.shift {
    display: block;
  }

  img {
    width: 100%;
    height: auto;
  }
}
.rst-content img {
  max-width: 100%;
  height: auto !important;
}
.rst-content div.figure {
  margin-bottom: 24px;
}
.rst-content div.figure.align-center {
  text-align: center;
}
.rst-content .section > img {
  margin-bottom: 24px;
}
.rst-content blockquote {
  margin-left: 24px;
  line-height: 24px;
  margin-bottom: 24px;
}
.rst-content .note .last, .rst-content .attention .last, .rst-content .caution .last, .rst-content .danger .last, .rst-content .error .last, .rst-content .hint .last, .rst-content .important .last, .rst-content .tip .last, .rst-content .warning .last, .rst-content .seealso .last {
  margin-bottom: 0;
}
.rst-content .admonition-title:before {
  margin-right: 4px;
}
.rst-content .admonition table {
  border-color: rgba(0, 0, 0, 0.1);
}
.rst-content .admonition table td, .rst-content .admonition table th {
  background: transparent !important;
  border-color: rgba(0, 0, 0, 0.1) !important;
}
.rst-content .section ol.loweralpha, .rst-content .section ol.loweralpha li {
  list-style: lower-alpha;
}
.rst-content .section ol.upperalpha, .rst-content .section ol.upperalpha li {
  list-style: upper-alpha;
}
.rst-content .section ol p, .rst-content .section ul p {
  margin-bottom: 12px;
}
.rst-content .line-block {
  margin-left: 24px;
}
.rst-content .topic-title {
  font-weight: bold;
  margin-bottom: 12px;
}
.rst-content .toc-backref {
  color: #404040;
}
.rst-content .align-right {
  float: right;
  margin: 0px 0px 24px 24px;
}
.rst-content .align-left {
  float: left;
  margin: 0px 24px 24px 0px;
}
.rst-content .align-center {
  margin: auto;
  display: block;
}
.rst-content h1 .headerlink, .rst-content h2 .headerlink, .rst-content h3 .headerlink, .rst-content h4 .headerlink, .rst-content h5 .headerlink, .rst-content h6 .headerlink, .rst-content dl dt .headerlink {
  display: none;
  visibility: hidden;
  font-size: 14px;
}
.rst-content h1 .headerlink:after, .rst-content h2 .headerlink:after, .rst-content h3 .headerlink:after, .rst-content h4 .headerlink:after, .rst-content h5 .headerlink:after, .rst-content h6 .headerlink:after, .rst-content dl dt .headerlink:after {
  visibility: visible;
  content: "\f0c1";
  font-family: FontAwesome;
  display: inline-block;
}
.rst-content h1:hover .headerlink, .rst-content h2:hover .headerlink, .rst-content h3:hover .headerlink, .rst-content h4:hover .headerlink, .rst-content h5:hover .headerlink, .rst-content h6:hover .headerlink, .rst-content dl dt:hover .headerlink {
  display: inline-block;
}
.rst-content .sidebar {
  float: right;
  width: 40%;
  display: block;
  margin: 0 0 24px 24px;
  padding: 24px;
  background: #f3f6f6;
  border: solid 1px #e1e4e5;
}
.rst-content .sidebar p, .rst-content .sidebar ul, .rst-content .sidebar dl {
  font-size: 90%;
}
.rst-content .sidebar .last {
  margin-bottom: 0;
}
.rst-content .sidebar .sidebar-title {
  display: block;
  font-family: "Roboto Slab", "ff-tisa-web-pro", "Georgia", Arial, sans-serif;
  font-weight: bold;
  background: #e1e4e5;
  padding: 6px 12px;
  margin: -24px;
  margin-bottom: 24px;
  font-size: 100%;
}
.rst-content .highlighted {
  background: #f1c40f;
  display: inline-block;
  font-weight: bold;
  padding: 0 6px;
}
.rst-content .footnote-reference, .rst-content .citation-reference {
  vertical-align: super;
  font-size: 90%;
}
.rst-content table.docutils.citation, .rst-content table.docutils.footnote {
  background: none;
  border: none;
  color: #999999;
}
.rst-content table.docutils.citation td, .rst-content table.docutils.citation tr, .rst-content table.docutils.footnote td, .rst-content table.docutils.footnote tr {
  border: none;
  background-color: transparent !important;
  white-space: normal;
}
.rst-content table.docutils.citation td.label, .rst-content table.docutils.footnote td.label {
  padding-left: 0;
  padding-right: 0;
  vertical-align: top;
}
.rst-content table.field-list {
  border: none;
}
.rst-content table.field-list td {
  border: none;
  padding-top: 5px;
}
.rst-content table.field-list .field-name {
  padding-right: 10px;
  text-align: left;
  white-space: nowrap;
}
.rst-content table.field-list .field-body {
  text-align: left;
  padding-left: 0;
}
.rst-content tt {
  color: black;
}
.rst-content tt big, .rst-content tt em {
  font-size: 100% !important;
  line-height: normal;
}
.rst-content tt .xref, a .rst-content tt {
  font-weight: bold;
}
.rst-content a tt {
  color: #2980b9;
}
.rst-content dl {
  margin-bottom: 24px;
}
.rst-content dl dt {
  font-weight: bold;
}
.rst-content dl p, .rst-content dl table, .rst-content dl ul, .rst-content dl ol {
  margin-bottom: 12px !important;
}
.rst-content dl dd {
  margin: 0 0 12px 24px;
}
.rst-content dl:not(.docutils) {
  margin-bottom: 24px;
}
.rst-content dl:not(.docutils) dt {
  display: inline-block;
  margin: 6px 0;
  font-size: 90%;
  line-height: normal;
  background: #e7f2fa;
  color: #2980b9;
  border-top: solid 3px #6ab0de;
  padding: 6px;
  position: relative;
}
.rst-content dl:not(.docutils) dt:before {
  color: #6ab0de;
}
.rst-content dl:not(.docutils) dt .headerlink {
  color: #404040;
  font-size: 100% !important;
}
.rst-content dl:not(.docutils) dl dt {
  margin-bottom: 6px;
  border: none;
  border-left: solid 3px #cccccc;
  background: #f0f0f0;
  color: gray;
}
.rst-content dl:not(.docutils) dl dt .headerlink {
  color: #404040;
  font-size: 100% !important;
}
.rst-content dl:not(.docutils) dt:first-child {
  margin-top: 0;
}
.rst-content dl:not(.docutils) tt {
  font-weight: bold;
}
.rst-content dl:not(.docutils) tt.descname, .rst-content dl:not(.docutils) tt.descclassname {
  background-color: transparent;
  border: none;
  padding: 0;
  font-size: 100% !important;
}
.rst-content dl:not(.docutils) tt.descname {
  font-weight: bold;
}
.rst-content dl:not(.docutils) .optional {
  display: inline-block;
  padding: 0 4px;
  color: black;
  font-weight: bold;
}
.rst-content dl:not(.docutils) .property {
  display: inline-block;
  padding-right: 8px;
}
.rst-content .viewcode-link, .rst-content .viewcode-back {
  display: inline-block;
  color: #27ae60;
  font-size: 80%;
  padding-left: 24px;
}
.rst-content .viewcode-back {
  display: block;
  float: right;
}

@media screen and (max-width: 480px) {
  .rst-content .sidebar {
    width: 100%;
  }
}
span[id*='MathJax-Span'] {
  color: #404040;
}
